#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2003,2004 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
# @(#)46   1.264.1.2   src/csm/hacsm/pm/HAMS.pm.perl, hacsm, csm_rbea, rbeas011a 12/8/04 13:39:22
#
#####################################################################
#                                                                   #
# Module: HAMS.pm                                                   #
#                                                                   #
#-------------------------------------------------------------------#
#                                                                   #
# Description: This module contains Perl subroutines used by HAMS   #
#                                                                   #
# Subroutines:                                                      #
#                                                                   #
#####################################################################

package HAMS;

BEGIN{
    #print "env = $ENV{'CSM_HAMS_DEBUG'}\n";
    if($ENV{'CSM_HAMS_DEBUG'}){ $::HAMS_DEBUG=1; $::VERBOSE=1; }
    $ENV{'CSM_HAMS_CONTROL'}=1;
    $::csmpm = $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm'; #for development use
    $ENV{'POSIXLY_CORRECT'}=1;
    $HAMS::TIMEOUT = 10;
    $HAMS::LONGTIME = 90;
}

use strict;
use Fcntl qw(:flock);
use File::Basename;
use File::Path;	# Provides mkpath()
use Getopt::Long;
use lib $::csmpm;
use CLOG;
use InstallKRB5Utils;
use NetworkUtils;

#TODO: error checking
#TODO: fix document. a : is needed after hostname in example def file


END{
  # get rid of KRB5 tgt if gotten
  my $exit = $?;
  HAMS->printDebug("K5ticket = $::krb5_tgt\n");
  if ($::krb5_tgt==1) {
    NodeUtils->runcmd("/opt/csm/csmbin/k5destroytgt", -1);
  }
  exit $exit;
}

#######################################################################
# loadGlobals
#######################################################################
sub loadGlobals{
  my ($class, $nocheck) = @_; #no check for skipping over the HAMODE value
  my $exit_code =0;
  #GLOBAL VARIABLES
  $::MSGCAT = 'hams.cat';
  $::MSGMAPPATH = "$::csmroot/msgmaps";
  $::MSGSET = 'hams';
  $::DSH = "/opt/csm/bin/dsh";
  $::DOMAINNAME = "CSM_HAMS";
  $::RPDOMAIN = "CSM_HAMS";
  $::MOUNT_POINT = "/var/opt/hams/mnt";
  $::TB_NAME = "HAMS_SCSI";
  $::SCRIPTS_DIR = "/opt/csm/hams/scripts";
  $::HAMS_LOG = "/var/log/csm/hams/hams.log";
  $ENV{'CT_MANAGEMENT_SCOPE'}=1;
  $ENV{'CSM_HAMS_CONTROL'}=1;
  #HAMS->onlineWaitSrc("IBM.DMSRM");
  my $output = NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 lsrsrc-api -i -s IBM.DmsCtrl::::RemoteShell::SetupRemoteShell::RemoteCopyCmd::HAMode::SetupKRB5", 0);
  if($::RUNCMD_RC!=0){ $exit_code=3; }
  ($::REMOTE_SHELL, $::SETUP_REMOTE_SHELL, $::REMOTECOPYCMD, $::HAMODE, $::SETUPKRB5) = split "::", $output;
  if(!$nocheck && $::HAMODE==0 && !($::START || $::ACCEPT_LICENSE)){
    print "HAMS has not been configured on this machine. Run \"hams -s\" to start and configure HAMS.\n";
    exit 2;
  }
  #$::REMOTE_SHELL = InstallUtils->getRemoteShell();
  #$::SETUP_REMOTE_SHELL = InstallUtils->getSetupRemoteShell();
  HAMS->printDebug("$::REMOTE_SHELL, $::SETUP_REMOTE_SHELL, $::HAMODE\n");
  
  $::HAMSDEF = "/var/opt/hams/hamsdef";

  my $osname         = $::PLTFRM;
  my $distro_name    = NodeUtils->get_DistributionName;     #requires no cmds
  my $distro_version = NodeUtils->get_DistributionVersion;  #requires LSLPP
  my $hw_arch = InstallUtils->get_PkgArchitecture;  #requires UNAME for Linux
  my $csm_version = "1.4.0";
  %::msOSDefs= InstallUtils->get_OSDefs($osname, $distro_name, $distro_version, $hw_arch, $csm_version);
  if($hw_arch=~ m/ppc/ && $osname =~ m/Linux/){
    $::PLINUX=1;
  }

  #ENVIRONMENT VARIABLES
  $ENV{'CT_MANAGEMENT_SCOPE'}=1;
  $::krb5_tgt =0;
  $::KRB5CCNAME="";
  $::KRB5CCNAME=InstallKRB5Utils->getKRB5tgt;
  HAMS->printDebug("k5 ticket = $::KRB5CCNAME\n");
  if ($::KRB5CCNAME ne "") {
    $ENV{'KRB5CCNAME'} = $::KRB5CCNAME;
    $::krb5_tgt=1;
  }
  $ENV{'CT_MANAGEMENT_SCOPE'}=2;
  if($exit_code!=0 && !$nocheck){
    exit $exit_code;
  }
  
}

#######################################################################
# getUsage
#######################################################################
sub getUsage{
    if(!@ARGV){ HAMS->pUsage(); }
    
    $Getopt::Long::ignorecase=0; #Checks case in GetOptions
    Getopt::Long::Configure ("bundling"); #allows short command line options to be grouped (e.g. -av)    
    
    if(!GetOptions( 
		"h"          => \$::HELP,
		"v|V"        => \$::VERBOSE,
		"s|start"  => \$::START,
		"S|stop"     => \$::STOP,
		"l"          => \$::LIST,
		"m"          => \$::HAMS_MOVE,
		"i"          => \$::INSTALL,
		"d=s"        => \$::INSTALLDIR,
		"p|pause"    => \$::PAUSE,
		"F"          => \$::OUTPUT_FILE,
		"c"          => \$::COPY_LOCAL,
	        "L=s"          => \$::ACCEPT_LICENSE,
		   "r|restore"  => \$::HAMS_RESTORE,
		)){
	HAMS->pUsage();
		    }

    if($::HELP){ HAMS->pUsage(); }
    if($::HAMS_RESTORE && !$::STOP){ HAMS->pUsage(); }
    my $count =0;
    if($::START){ $count++; }
    if($::STOP){ $count++; }
    if($::HAMS_MOVE){ $count++; }
    if($::LIST){ $count++; }
    if($::COPY_LOCAL){ $count++; }
    if($::PAUSE){ $count++; }
    if($::ACCEPT_LICENSE){ $count++; }
    if($count > 1){ HAMS->pUsage(); }
    if($::START && !@ARGV){ $::START = "/opt/csm/hams/hamsdef"; }
    elsif($::START && @ARGV){ 
      $::START = $ARGV[0]; 
      if(!-e "$::START"){ 
	NodeUtils->message('E1', 'EMsgFileDoesNotExist', $::START);
      }
    }
    if(($::COPY_LOCAL) && (NodeUtils->isAIX())){
      NodeUtils->message('E1', 'EMsgAIXNoCFlag');
    }
    if($::OUTPUT_FILE){ undef $::VERBOSE; } #turn off -v if -F
    
    if($::ACCEPT_LICENSE){ 
      if(scalar @ARGV > 1){ HAMS->pUsage(); }
      @::LICENSE = ($::ACCEPT_LICENSE, $ARGV[0]);
    }
 }
  

sub pUsage{
  NodeUtils->message('I', 'IMsgHAMSUsage');
  if($::HELP){ exit 0; }
  exit 1;
}

#######################################################################
# getAttributes
#
# Attribute format
#    Hostname: 
#    IsPrimary=<1 for primary, 0 for backup> 
#    PartitionDeviceName=<Partition for storing shared CSM data> 
#    FileSystemType=<Partition file system type, e.g. ext3 or jfs> 
#    VolumeGroup
#    DiskDeviceName
#    SCSILockID=<The ID of SCSI device to be used for quorum determination> 
#    SCSILockLUN=<The LUN of SCSI device to be used for quorum determination> 
#    SCSILockHOST=<The HOST of SCSI device to be used for quorum determination> 
#    SCSILockCHAN=<The CHAN of SCSI device to be used for quorum determination> 
#    NIMNetworkInterface=<AIX only. The network interface to be used by the NIM master> 
#    PublicIPAddress=<Optional. This is the IP alias that will be used for IPAT of the primary s connection to the public VLAN. 
#                    This value should be the same for both management servers.> 
#   PublicIPNetmask=<Optional. The netmask for the IP alias specified above. This value should be the same for both management servers.> 
#   PublicIPAdapter=<Optional. The ethernet adapter that the Public IP address should be aliased onto. This value can be unique for each management server> 
#   ClusterVLANAdapter=<Optional. The ethernet adapter that is connected to the cluster VLAN.> 
#   ManagementVLANAdapter=<Optional. The ethernet adapter that is connected to the management VLAN.>
#
# Input: Filename
# Output: reference to three hashes of ms values
#######################################################################
sub getAttributes{
    my ($class, $filename) = @_;
    HAMS->printDebug("inside getAttributes\n");
    #TODO: attribute checking
    my (%both);
    if((!-f $filename) && ($::HAMODE != 0) ){
		print "file $filename does not exist or is the wrong type: $!\n";
		exit 1;
    }
    open(DEFFILE, $filename) or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $filename, $!);
    my $line;
    my $stanza = 0;
    while($line = <DEFFILE>){
	chomp $line;
	$line =~ s/\s+//g;
	if($line !~ m'^#' && $line =~ m/\w/){ #if its not a comment and its not blank
		my $setHostname=0;
	    #$line =~ s/\s+//g;
	    if($line =~ m/(\S+):(.*)/){ #this is the hostname
	    	if($2 !~ m/\w/){
	    		HAMS->printDebug("HOSTNAME_LINE = |$line|\n");
				$stanza++;

				if($stanza > 2){
					print "only 2 stanzas allowed!";
					exit 1;
				}
			        my $hostname = NodeUtils->tryHost($1);
				$both{$stanza}{'hostname'}=$hostname;
				HAMS->printDebug("stanza $stanza has hostname $hostname\n");
				$setHostname=1;
			}
	    }
	    if($stanza && ($setHostname==0)){
			my ($attr, $value) = split(/=/, $line, 2);
			if($value ne ""){
			    $both{$stanza}{$attr}=$value;
			}
	    }
	}
    }
    close DEFFILE or NodeUtils->message('E2', 'EMsgPerlFailed', "close", $filename, $!);
    
    my ($primary, $backup);
    foreach my $num (keys %both){
	if($both{$num}{'IsPrimary'}==1){
	    $primary = $both{$num};
	}
	else{
	    $backup = $both{$num};
	}
    }
    if(($primary eq "") || ($backup eq "")){
		print "Invalid hamsdef file: $filename. Please ensure both hostnames are defined.\n";
		exit 1;
	}
    if ($$primary{'PartitionDeviceName'} !~ /^\/dev\//g || $$backup{'PartitionDeviceName'} !~ /^\/dev\//g){
        NodeUtils->message('E1', 'EMsgInvalidSharedDisk', "PartitionDeviceName");
    }
    if (NodeUtils->isAIX() && ($$primary{'PhysicalVolume'} !~ /\S+/g || $$backup{'PhysicalVolume'} !~ /\S+/g)){
        NodeUtils->message('E1', 'EMsgInvalidSharedDisk', "PhysicalVolume");
    }
    if (NodeUtils->isLinux() && ($$primary{'SCSILockID'} !~ /\S+/g || $$primary{'SCSILockLUN'} !~ /\S+/g || $$primary{'SCSILockHOST'} !~ /\S+/g || $$primary{'SCSILockCHAN'} !~ /\S+/g || $$backup{'SCSILockID'} !~ /\S+/g || $$backup{'SCSILockLUN'} !~ /\S+/g || $$backup{'SCSILockHOST'} !~ /\S+/g || $$backup{'SCSILockCHAN'} !~ /\S+/g)){
        NodeUtils->message('E1', 'EMsgInvalidSharedDisk', "SCSILockID, SCSILockLUN, SCSILockHOST, SCSILockCHAN");
    }
    if(($$primary{'PublicIPAddress'}=~ m/\d/) || ($$backup{'PublicIPAddress'}=~ m/\d/)){
	$::IPAT=1;
    }


    my @machines = ($$primary{'hostname'}, $$backup{'hostname'});
    if(!$::localhost){ $::localhost = InstallUtils->determineMS(\@machines); }
    my $current;
    if($::localhost eq $$primary{'hostname'}){ $current = $primary; } #on primary
    else{ $current = $backup; } #on backup

    NodeUtils->message('LI', 'IMsgHamsdefInfo', $$primary{'hostname'}, $$backup{'hostname'}, $$current{'hostname'});
    return($primary, $backup, $current);
}

#######################################################################
# setupRemoteShell
# This routine sets up the remote shell from ms1 -> ms2
#
#######################################################################
sub setupRemoteShell{
  my ($class, $ms1, $ms2) = @_;
  my $cmd;
  if($::localhost eq $ms1){
    my $reachable =0;
    my @machines = ($ms2);
    my $node_file = InstallUtils->make_node_list_file(\@machines); #put all dest nodes in tmp file
    
    (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
    if ($cmd_basename ne "rsh" && (InstallUtils->supportedSSHversion())){ #can do SSH with options
      $cmd = "$::REMOTESHELL_EXPECT -ds $node_file";
    }
    else{
      $cmd = "$::REMOTESHELL_EXPECT -d $node_file";   
    }
    my @output = NodeUtils->runcmd($cmd, -1);
    InstallUtils->close_delete_file($::NODE_LIST_FILE, $node_file); #delete node file
    foreach my $line (@output){
      chomp $line;
      my @parts = split ":", $line;
      if(($parts[1] =~ m/test.success/) && ($parts[0] eq $ms2)){ #this node is reachable
	$reachable=1;
      }
    }
    if(!$reachable && $::SETUP_REMOTE_SHELL){ #SETUP REMOTE SHELL
      InstallUtils->checkHOMEforSSHperm();
      #determine if we can setup this remote shell
      (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
      if ($cmd_basename ne "rsh"){
	#check if its a supported version of OpenSSH
	if(InstallUtils->supportedSSHversion()){
	  $::SSH_SUPPORT=1;
	  HAMS->setupLocalRemoteSSH($ms1, $ms2);
	  #TODO: report error if any FAILED
	}
	else{ #can't setup this remote protocol
	  NodeUtils->message('O', 'IMsgNoSetupRemoteProtocol', $::REMOTE_SHELL);
	}
      }
      else{ #it is rsh
	HAMS->setupLocalRemoteRSH($ms1, $ms2);
	#TODO: report error if any FAILED
      }
    }
    if($::SETUPKRB5 >= 8){ #run pop5
      #put this machines id on the other ms
      my $ms1id = NodeUtils->getRSCThostname;
      $ms1id = lc($ms1id);
      $ms1id = "ctsrcmdp." . $ms1id;
      NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $ms2 \"/opt/csm/csmbin/popk5login -p $ms1id\"", 0);
    }
  }
  elsif($::localhost eq $ms2){ #check the remote shell from the other machine to this one
    my $reachable=0;
    (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
    if ($cmd_basename ne "rsh" && (InstallUtils->supportedSSHversion())){ #can do SSH with options
      $cmd = "$::DSH -t $HAMS::LONGTIME -vn $ms1 $::REMOTESHELL_EXPECT -dsn $ms2";
    }
    else{
      $cmd = "$::DSH -t $HAMS::LONGTIME -vn $ms1 $::REMOTESHELL_EXPECT -dn $ms2";   
    }
    HAMS->printDebug("RUNNING: $cmd\n");
    my @output = `$cmd 2>&1`;
    foreach my $line (@output){
      HAMS->printDebug($line);
      chomp $line;
      my @parts = split ":", $line;
      if(($parts[2] =~ m/test.success/) && ($parts[1] =~ $ms2)){ #this node is reachable
	$reachable=1;
	HAMS->printDebug("machine $ms2 is reachable\n");
      }
    }
    if(!$reachable && $::SETUP_REMOTE_SHELL){
      HAMS->printDebug("machine $ms2 is NOT reachable\n");
      (my $cmd_basename = $::REMOTE_SHELL) =~ s:^.*/::;
      if ($cmd_basename ne "rsh"){
	if(InstallUtils->supportedSSHversion()){
	  HAMS->setupRemoteLocalSSH($ms1, $ms2);
	}
      }
      else{ #it is rsh
	HAMS->setupRemoteLocalRSH($ms1, $ms2);		
      }
    }
    if($::SETUPKRB5 >= 8){ #run pop5
      #put the other machines id on this ms
      my @output = NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $ms1 \"/usr/sbin/rsct/bin/lsnodeid\"", -1);
      my ($nothing, $ms1id) = split ": ", $output[0];
      $ms1id = lc($ms1id);
      $ms1id = "ctsrcmdp." . $ms1id;
      NodeUtils->runcmd("/opt/csm/csmbin/popk5login -p $ms1id", 0);
    }
  }
}

#######################################################################
# stopHAMSDomain
# TODO: finish
# TODO: put in full pathname of commands
#######################################################################
sub stopHAMSDomain{
  my ($class, $primary, $backup, $pvals, $bvals, $cvals) = @_;
  NodeUtils->runcmd("/usr/bin/chrg -o Offline Active_MS",-1);
  NodeUtils->runcmd("/usr/bin/chrg -o Offline Inactive_MS",-1);
  my $i=0;
  while(HAMS->waitRGOffline("Active_MS") && ($i <100)){ sleep 2; $i++; }
  while(HAMS->waitRGOffline("Inactive_MS")&& ($i <200)){ sleep 2; $i++;}
  #while(HAMS->waitAppOffline($pvals, $bvals, "Save_CSM_Data")&& ($i <100)){ sleep 2; $i++;}
  HAMS->printDebug("All RGs offline\n");
  HAMS->removeCHARMRes($primary, $backup);

  HAMS->mountSharedDisk($pvals, $bvals, $cvals);
  if($::HAMS_RESTORE){
    my $rc = HAMS->migrateBack($pvals, $bvals, $cvals);
    HAMS->unMountSharedDisk($pvals, $bvals, $cvals);
  }
  NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"CT_MANAGEMENT_SCOPE=1 /usr/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::0\"", 0);
  sleep 2;
  #check to see if $::RPDOMAIN is the current one
  my @output = NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 /usr/bin/lsrsrc-api -D ':|:' -s IBM.PeerDomain::\"Name IN ('$::RPDOMAIN')\"::Name::OpState", -1);
  if($::RUNCMD_RC !=0){
    NodeUtils->message('E', 'EMsgNoRPD', $::RPDOMAIN);
  }
  else{
    NodeUtils->runcmd("/usr/bin/stoprpdomain $::RPDOMAIN", -1); 
    sleep 7;
    NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"CT_MANAGEMENT_SCOPE=1 /usr/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::0\"", -1);
    NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"rmrpdomain -fV $::RPDOMAIN\"", -1);
    sleep(2);
    my $rc=1;
    while($rc){
      sleep 2;
      my $ver = $::VERBOSE;
      $::VERBOSE=0;
      NodeUtils->runcmd("/opt/csm/bin/csmconfig",-1);
      print ".";
      $::VERBOSE = $ver;
      $rc = $::RUNCMD_RC;
    }
    print ".\n";
    sleep 2;
  }
  if(!$::HAMS_RESTORE){
   $::HAMS_MOUNT=0;
   HAMS->mountSharedDisk($pvals, $bvals, $cvals);
  }
  NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 /usr/bin/refrsrc-api -c IBM.DmsCtrl");
  NodeUtils->message('I', 'IMsgHAMSCleanedUp');
}

# start the rsct peer domain
sub createRPDomain
{
    my ($class, $primary, $backup) = @_;
    my $online =0;
    NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"/usr/bin/chrsrc-api -i -s IBM.DmsCtrl::::HAMode::3\"");
    #my $output = NodeUtils->runcmd('/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerNode::"NodeNameList IN (\'$primary\',\'$backup\')"::\'*p0x0020\'', -1);
    my $outref = NodeUtils->runcmd("/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerNode::\"NodeNameList IN (\'$primary\',\'$backup\')\"::NodeNameList", -1,1);
    
    if($::RUNCMD_RC==0){
		#there is an online peer domain containing at least 1 ms
		my $found_p =0;
		my $found_b =0;
		foreach my $line (@$outref){
			#print "$line\n";
			my ($nothing, $hostname) = split(/:\|:/, $line);
			$hostname =~ s/{|}//g;
			if($hostname eq $primary){ $found_p=1; }
			elsif($hostname eq $backup){ $found_b=1; }
		}
		if(!($found_p && $found_b)){ #one of them is not in the peer domain
			print "There is an existing peer domain, but both management servers are not in it.\n";
			exit 1;
		}
		else{ #both are in there, get domain name
			$outref = NodeUtils->runcmd("/usr/bin/lsrsrc-api -D ':|:' -s IBM.PeerDomain::::Name::OpState",-2,1);
			foreach my $line (@$outref){
				my ($name, $state) = split(/:\|:/, $line);
				if($state==1){
		 			$::RPDOMAIN=$name;
		 			$online =1;
		 			print "Peer domain is already created and online: $::RPDOMAIN\n";
		 			last;
				}
			}
		}
	}
    else{ 
		
		#There may be an offline RPD
		my $outref = NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 /usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerDomain::\"Name IN (\'$::RPDOMAIN')\"", -1,1);
		if($::RUNCMD_RC!=0){ #no existing peer domain
			NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup CT_MANAGEMENT_SCOPE=2 /usr/bin/preprpnode $primary $backup");
			NodeUtils->runcmd("/usr/bin/mkrpdomain $::RPDOMAIN $primary $backup");
    	}
    	if(!$online){ #start the peer domain
		    NodeUtils->runcmd("/usr/bin/startrpdomain $::RPDOMAIN");
		    #wait until domain is online
		    HAMS->waitRPD();
		   # system("lsrpnode");
    	}
	}
	
}


###########################################################
# setupLocalRemoteSSH
#	input: \@target_nodes
###########################################################
sub setupLocalRemoteSSH{
	my ($class, $localhost, $machine) = @_;
	my @machines = ($machine);
	InstallUtils->rmNodesKnownHosts(\@machines);
	NodeUtils->runcmd("$::REMOTESHELL_EXPECT -k"); #make the OpenSSH keys
	InstallUtils->cpSSHFiles();
	my $SSHdir = "/csminstall/csm/config/.ssh";
	open (FILE, ">$SSHdir/copy.perl") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", "$SSHdir/copy.perl", $!);
	
	print FILE "#!/usr/bin/perl
my \@root = split ':', (`/bin/grep ^root /etc/passwd 2>&1`);
my \$home = \$root[5];
umask(0077);
\$dest_dir = \"\$home/.ssh/\";
if (! -d \"\$dest_dir\" ) {
    # create a local directory
    \$cmd = \"$::MKDIR -p \$dest_dir\";
    system(\"\$cmd\"); 
    chmod 0700, \$dest_dir;
}
`$::CAT /tmp/.ssh/authorized_keys >> \$home/.ssh/authorized_keys 2>&1`;
`$::CAT /tmp/.ssh/authorized_keys2 >> \$home/.ssh/authorized_keys2 2>&1`;
`$::RM -f /tmp/.ssh/authorized_keys 2>&1`;
`$::RM -f /tmp/.ssh/authorized_keys2 2>&1`;
`$::RM -f /tmp/.ssh/copy.perl 2>&1`;
rmdir(\"/tmp/.ssh\");";
	close FILE or NodeUtils->message('E2', 'EMsgPerlFailed', "open", "$SSHdir/copy.perl", $!);
	chmod 0744, "$SSHdir/copy.perl";
	#set an ENV var if more than 10 nodes
	system("$::REMOTESHELL_EXPECT -s $machine"); #has to be a system call instead of runcmd for the prompt
	my $exit = $? >> 8;
	if( $exit !=0 ){
	  NodeUtils->runcmd("$::REMOTESHELL_EXPECT -t $::REMOTE_SHELL $machine", -1);
	  if($::RUNCMD_RC !=0 ){
	    NodeUtils->message('E2', 'EMsgRemoteShellSetupFailed', $::REMOTE_SHELL, $localhost, $machine);
	  }
	}
	system("stty echo");
}

###########################################################
# setupLocalRemoteRSH
#	input: \@target_nodes
###########################################################
sub setupLocalRemoteRSH{
  my ($class, $localhost, $machine) = @_;

  if( NodeUtils->isLinux() ){
    #RH or SuSE
    #RH = $::REMOTESHELL_EXPECT -rL $MS $machine
    system("$::REMOTESHELL_EXPECT -rL $localhost $machine"); #has to be a system call instead of runcmd for the prompt
    #TODO: SuSE = $::REMOTESHELL_EXPECT -rS $MS $machine
  }
  elsif( NodeUtils->isAIX()){
    #AIX = $::REMOTESHELL_EXPECT -rA $MS $machine
    system("$::REMOTESHELL_EXPECT -rA $localhost $machine"); #has to be a system call instead of runcmd for the prompt
  }
  else{
    #TODO: NodeUtils->message('E', 'EMsgUnsupportedOS', $n, $os );
  }
  my $exit = $?>>8;
  if( $exit !=0 ){
    NodeUtils->runcmd("$::REMOTESHELL_EXPECT -t $::REMOTE_SHELL $machine");
    if($::RUNCMD_RC !=0){
      NodeUtils->message('E2', 'EMsgRemoteShellSetupFailed', $::REMOTE_SHELL, $localhost, $machine);
    }
  }
  
  system("stty echo");
}

##########################################################
# setupRemoteLocalSSH
#	input: \@target_nodes
###########################################################
sub setupRemoteLocalSSH{
	my ($class, $remote, $local) = @_;
	#$cmd = "$::DSH -n $remote $::REMOTESHELL_EXPECT -k";
	NodeUtils->runcmd("$::DSH -t $HAMS::LONGTIME -vn $remote $::REMOTESHELL_EXPECT -k");
	my $home_dir = $ENV{'HOME'};
	if(!-d "/var/opt/hams"){ mkdir("/var/opt/hams"); }
	mkdir("/var/opt/hams/ssh");
	NodeUtils->runcmd("$::REMOTECOPYCMD $remote:$home_dir/.ssh/id\*.pub /var/opt/hams/ssh/.");
	NodeUtils->runcmd("$::CAT /var/opt/hams/ssh/identity.pub /var/opt/hams/ssh/id_rsa.pub /var/opt/hams/ssh/id_dsa.pub >> $home_dir/.ssh/authorized_keys");
	NodeUtils->runcmd("$::RM -rf /var/opt/hams/ssh");
	NodeUtils->runcmd("$::DSH -t $HAMS::LONGTIME -vn $remote $::REMOTESHELL_EXPECT -n $local");
}

###########################################################
# setupRemoteLocalRSH
#	input: \@target_nodes
###########################################################
sub setupRemoteLocalRSH{
  my ($class, $remote, $local) = @_;
  if(NodeUtils->isAIX()){
    InstallUtils->setupAIXRSH($remote);
  }
  else{
    InstallUtils->setupLinuxRSH($remote);
  }
}

###########################################################
# createHAMSResources
###########################################################
sub createHAMSResources{
  my ($class, $pvals, $bvals, $cvals, $skip_online) = @_;
  NodeUtils->runcmd("/usr/bin/chrg -o Offline Active_MS",-1);
  NodeUtils->runcmd("/usr/bin/chrg -o Offline Inactive_MS",-1);
  
  NodeUtils->runcmd("/usr/bin/rmrg -s \"Name IN ('Active_MS','Inactive_MS')\"", -1); 
  NodeUtils->runcmd("/usr/bin/rmrsrc -s \"Name IN ('HAMS_IPAT')\" IBM.ServiceIP", -1);
  NodeUtils->runcmd("/usr/bin/rmequ HAMS_NET_EQU", -1);

  HAMS->rmrsrc_apps($$pvals{'hostname'});
  HAMS->rmrsrc_apps($$bvals{'hostname'});
  
  NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $$pvals{'hostname'},$$bvals{'hostname'} \"$::RM -f /var/opt/hams/resources/*\"", -1);
  #make IBM.Application resources
  my $app_file = HAMS->createApplicationFile($pvals, $bvals);
  NodeUtils->runcmd("/usr/bin/mkrsrc -F $app_file IBM.Application");
  unlink($app_file);
  
  #make tiebreaker resources
  #see if the tiebreaker already exists
  my $output = NodeUtils->runcmd("/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.TieBreaker::\"Name IN ('$::TB_NAME')\"::Name",-1);
  if($::RUNCMD_RC == 0){ #TB DOES exist, remove it
    NodeUtils->runcmd("chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"Operator\"",-1);
    NodeUtils->runcmd("/usr/bin/rmrsrc -s \"Name IN ('$::TB_NAME')\" IBM.TieBreaker", -1); 
  }
  #Make TB
  if(NodeUtils->isLinux()){
    my $id =$$pvals{'SCSILockID'};
    my $lun =$$pvals{'SCSILockLUN'};
    my $host =$$pvals{'SCSILockHOST'};
    my $chan = $$pvals{'SCSILockCHAN'};
    my $id2 =$$bvals{'SCSILockID'};
    my $lun2 =$$bvals{'SCSILockLUN'};
    my $host2 =$$bvals{'SCSILockHOST'};
    my $chan2 = $$bvals{'SCSILockCHAN'};
    NodeUtils->runcmd("/usr/bin/mkrsrc IBM.TieBreaker Name=$::TB_NAME Type=SCSI NodeInfo='{[\"$$bvals{'hostname'}\",\"ID=$id2 LUN=$lun2 HOST=$host2,CHAN=$chan2\"],[\"$$pvals{'hostname'}\",\"ID=$id LUN=$lun HOST=$host,CHAN=$chan\"]}' HeartbeatPeriod=30");
    NodeUtils->runcmd("/usr/bin/chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"$::TB_NAME\"");
      }
  elsif(NodeUtils->isAIX()){
    my $device = $$pvals{'PhysicalVolume'};
    my $device2 = $$bvals{'PhysicalVolume'};
    if(defined $device && defined $device2){
      NodeUtils->runcmd("/usr/bin/mkrsrc IBM.TieBreaker Name=$::TB_NAME Type=DISK NodeInfo='{[\"$$bvals{'hostname'}\",\"DEVICE=$device2\"], [\"$$pvals{'hostname'}\",\"DEVICE=$device\"]}' HeartbeatPeriod=30");
      NodeUtils->runcmd("/usr/bin/chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"$::TB_NAME\"");
    }
  }

  HAMS->mkrg_mkrel($pvals, $bvals, $cvals);

  #change HAMode to 2
  NodeUtils->runcmd("/opt/csm/bin/csmconfig HAMode=2");
  
  if(!$skip_online){
      #Online the resource groups
      NodeUtils->runcmd("/usr/bin/chrg -o Online Active_MS");
      sleep 3;
      NodeUtils->runcmd("/usr/bin/chrg -o Online Inactive_MS");
      NodeUtils->runcmd("/opt/csm/bin/csmconfig HAMode=1");
  }
}

###########################################################
# waitRPD
###########################################################
sub waitRPD{
  my $online = 0;
  my $count =0; 
  #print "Waiting for RPD \"$::RPDOMAIN\" to come online";
  NodeUtils->message('I', 'IMsgWaitingForRPD', $::RPDOMAIN);
  while(($online==0) && ($count < 120)){
    if($count < 5){
      sleep 3;
      print "..";
    }
    elsif($count < 10){
      sleep 2;
      print ".";
    }
    else{ 
      sleep 1;
    }
    my @out = `/usr/bin/lsrsrc-api -D ':|:' -n -s IBM.PeerDomain::\"Name IN (\'$::RPDOMAIN\')\"::OpState 2>/dev/null`;
    foreach my $line (@out){
      my ($attr, $value) = split(/:\|:/, $line);
      if($value==1){
	$online =1;
	#print ".the RPD \"$::RPDOMAIN\" is now online.\n";
	NodeUtils->message('I', 'IMsgRPDOnline', $::RPDOMAIN);
      }
    }
    if(!$online){ print "."; }
    $count++;
    
  }
  if(!$online){ 
    NodeUtils->message('E1', 'EMsgNoOnlineRPD', $::RPDOMAIN);
    #return 1; 
  }
  return 0;
}

###########################################################
# mountMkdirs
###########################################################
sub mountMkdirs{
	my ($class, $filename, $primary, $backup) =@_;
	if(!-d "/var/opt/hams"){ mkdir("/var/opt/hams"); }
	#if(!-d "/var/opt/hams/mnt"){ mkdir("/var/opt/hams/mnt"); }
	if($filename ne $::HAMSDEF){
	  NodeUtils->runcmd("$::CP $filename $::HAMSDEF");
	}
	NodeUtils->runcmd("/opt/csm/bin/dcp -n $backup $filename $::HAMSDEF");
}

###########################################################
# onlineWaitSrc
###########################################################
sub onlineWaitSrc{
  my ($class, $resource, $wait_time) = @_;
  my $online = 0;
  if(!defined $wait_time){ $wait_time = 20; }
  my $count =0; 
  #check if online already
  my @out = NodeUtils->runcmd("LANG=C /usr/bin/lssrc -s $resource", -1);
  if(!$::RUNCMD_RC){
    my ($subsys, $group, $pid, $status) = split(' ', $out[1]);
    if (defined($status) && $status eq 'active'){
      $online =1;
    }
  }
  if(!$online){ #wait for it to com online
    NodeUtils->runcmd("/usr/bin/startsrc -s $resource");
    while(($online==0) && ($count < $wait_time)){
      if($count < 5){
	sleep 2;
      }
      else{ 
	sleep 1;
      }
      if($count==0){ NodeUtils->message('I', 'IMsgWaitingSRC', $resource); }
      my @out = `LANG=C /usr/bin/lssrc -s $resource`;
      my $rc = $? >>8;
      if(!$rc){
	my ($subsys, $group, $pid, $status) = split(' ', $out[1]);
	if (defined($status) && $status eq 'active'){
	  $online =1;
	  print "\n";
	  NodeUtils->message('I', 'IMsgSRCActive', $resource);
	}
      }
      if(!$online){ print "."; }
      $count++;
    }
  }
  if(!$online){ return 1; }
  return 0;
}

###########################################################
# createApplicationFile
#    returns filename
#
# X Active_CSM_Dep IBM.Application Runs user scripts: active_csm_start.pre and
#               active_csm_stop.post
#
# X Active_CSM IBM.Application Starts, stops and monitors IBM.DMSRM
#
# X Active_CSM_Req IBM.Application Runs user scripts: active_csm_start.post and
#               active_csm_stop.pre
#
# X Shared_Disk_Dep IBM.Application Runs user scripts: shared_disk_start.pre and
#               shared_disk_stop.post
#
# X Shared_Disk IBM.Application (unless DiskRM is available)
#               Mounts, unmounts, and monitors the connection
#               to the shared disk.
#
# X Shared_Disk_Req IBM.Application Runs user scripts: shared_disk_start.post and
#               shared_disk_stop.pre
#
# X Save_CSM_Data IBM.Application Starts, stops, and monitors the ERRM conditions
#               and responses that save the CSM and RSCT data
#               to disk.
#
# X Public_IP_Dep IBM.Application Runs user scripts: public_ip_start.pre and
#               public_ip_stop.post
#
# X Public_IP_Req IBM.Application Runs user scripts: public_ip_start.post and
#               public_ip_stop.pre
#
# X Inactive_CSM IBM.Application Starts, stops and monitors IBM.DMSRM
#
# X Gather_CSM_Nodelist IBM.Application Starts, stops and monitors the condition and
#               responses that replicate the primarys nodelist and
#               setup CtSec security.
#
###########################################################
sub createApplicationFile{
	my ($class, $pvals, $bvals) = @_;
	
	my $filename = HAMS->getRandFilename();
	my $nodestring =  "'{\"$$pvals{'hostname'}\",\"$$bvals{'hostname'}\"}'";

	#TODO: open and close error checking
	open(APPFILE, ">$filename") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $filename, $!);
		
		print APPFILE <<"END_OF_FILE";
PersistentResourceAttributes::
	resource 1:
		Name="Active_CSM"
		StartCommand="/opt/csm/csmbin/hams/Active_CSM --start" 
		StartCommandTimeout=300
		StopCommand="/opt/csm/csmbin/hams/Active_CSM --stop"
		StopCommandTimeout=120
		MonitorCommand="/opt/csm/csmbin/hams/Active_CSM --monitor"
		MonitorCommandPeriod=20
		MonitorCommandTimeout=20
		UserName="root"
		ProtectionMode=1
		NodeNameList=$nodestring
	resource 2:
		Name="Inactive_CSM"
		StartCommand="/opt/csm/csmbin/hams/Inactive_CSM --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Inactive_CSM --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Inactive_CSM --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 3:
		Name="Active_CSM_Dep"
		StartCommand="/opt/csm/csmbin/hams/Active_CSM_Dep --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Active_CSM_Dep --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Active_CSM_Dep --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 4:
		Name="Active_CSM_Req"
		StartCommand="/opt/csm/csmbin/hams/Active_CSM_Req --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Active_CSM_Req --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Active_CSM_Req --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 5:
		Name="Shared_Disk_Dep"
		StartCommand="/opt/csm/csmbin/hams/Shared_Disk_Dep --start"
		StartCommandTimeout=120
		StopCommand="/opt/csm/csmbin/hams/Shared_Disk_Dep --stop"
		StopCommandTimeout=120
		MonitorCommand="/opt/csm/csmbin/hams/Shared_Disk_Dep --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 6:
		Name="Shared_Disk"
		StartCommand="/opt/csm/csmbin/hams/Shared_Disk --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Shared_Disk --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Shared_Disk --monitor"
		MonitorCommandPeriod=35
		MonitorCommandTimeout=30
		UserName="root"
		NodeNameList=$nodestring
	resource 7:
		Name="Shared_Disk_Req"
		StartCommand="/opt/csm/csmbin/hams/Shared_Disk_Req --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Shared_Disk_Req --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Shared_Disk_Req --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 8:
		Name="Save_CSM_Data"
		StartCommand="/opt/csm/csmbin/hams/Save_CSM_Data --start"
		StartCommandTimeout=300
		StopCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Save_CSM_Data --stop"
		MonitorCommand="/opt/csm/csmbin/hams/Save_CSM_Data --monitor"
		MonitorCommandPeriod=20
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 9:
		Name="Public_IP_Dep"
		StartCommand="/opt/csm/csmbin/hams/Public_IP_Dep --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Public_IP_Dep --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Public_IP_Dep --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 10:
		Name="Public_IP_Req"
		StartCommand="/opt/csm/csmbin/hams/Public_IP_Req --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Public_IP_Req --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Public_IP_Req --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 11:
		Name="Gather_CSM_Nodelist"
		StartCommand="/opt/csm/csmbin/hams/Gather_CSM_Nodelist --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Gather_CSM_Nodelist --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Gather_CSM_Nodelist --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 12:
		Name="Active_VLAN_Connection"
		StartCommand="/opt/csm/csmbin/hams/Active_VLAN_Connection --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Active_VLAN_Connection --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Active_VLAN_Connection --monitor"
		MonitorCommandPeriod=20
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
	resource 13:
		Name="Inactive_VLAN_Connection"
		StartCommand="/opt/csm/csmbin/hams/Inactive_VLAN_Connection --start"
		StartCommandTimeout=60
		StopCommand="/opt/csm/csmbin/hams/Inactive_VLAN_Connection --stop"
		StopCommandTimeout=60
		MonitorCommand="/opt/csm/csmbin/hams/Inactive_VLAN_Connection --monitor"
		MonitorCommandPeriod=60
		MonitorCommandTimeout=20
		UserName="root"
		NodeNameList=$nodestring
END_OF_FILE
	close APPFILE or NodeUtils->message('E2', 'EMsgPerlFailed', "close", $filename, $!);

	return $filename;

}

##########################################################
# getRandFilename
#    returns filename
###########################################################
sub getRandFilename{
	my $file = "/tmp/hams_$$";
    while(-e $file){
    	$file = InstallUtils->CreateRandomName($file);
    }
    return $file;
}

###########################################################
# migrateCSMFiles
#Initial migration of CSM data to shared disk on active management server HA CSM will create 
#the following subdirectories: 
#   /var/opt/hams/mnt/csminstall 
#   /var/opt/hams/mnt/etc 
#   /var/opt/hams/mnt/var 
#Once the above directories have been created HA CSM will recursively copy the existing data from 
#/csminstall, /etc/opt/csm, and /var/opt/csm onto the shared disk. Once all of the data has been 
#copied, all files will be removed from the aforementioned directories and they will be used as 
#symbolic links. On the active management server, the following symbolic links will be made. 
#These symbolic links will allow the CSM code to access its files in the regular locations, 
#while making sure they are stored on the shared disk. 
#/csminstall -> /var/opt/hams/mnt/csminstall 
#/etc/opt/csm -> /var/opt/hams/mnt/etc 
#/var/opt/csm -> /var/opt/hams/mnt/var
###########################################################
sub migrateCSMFiles{
    my ($class, $pvals, $bvals, $cvals) = @_;

    HAMS->mountSharedDisk($pvals, $bvals, $cvals, 1); #1 = don't mountLinux local

    if(NodeUtils->isAIX()){ #use sym links for AIX 
      my $local_dir = "/var/opt/hams";
      my @dirs = ("csminstall.local", "etc.local", "var.local");
      foreach my $d (@dirs){
	if(!-d "$local_dir/$d"){ mkdir("$local_dir/$d"); }
      }

      #make sub dirs
      my @dirs = ("csminstall", "etc", "var", "usr");
      foreach my $d (@dirs){
	if(!-d "$::MOUNT_POINT/$d"){ mkdir("$::MOUNT_POINT/$d"); }
      }
      HAMS->moveDir("/csminstall", "$::MOUNT_POINT/csminstall", 1, $cvals);
      HAMS->moveDir("/etc/opt/csm", "$::MOUNT_POINT/etc", 1, $cvals);
      HAMS->moveDir("/var/opt/csm", "$::MOUNT_POINT/var", 1, $cvals);
    }
    else{ #use remounted directories for Linux
      #make sub dirs
      my @dirs = ("csminstall", "etc", "var", "usr");
      foreach my $d (@dirs){
	if(!-d "$::MOUNT_POINT/$d"){ mkdir("$::MOUNT_POINT/$d"); }
      }
      HAMS->moveMountDir("/csminstall", "$::MOUNT_POINT/csminstall", 1, $pvals, $bvals, $cvals);
      HAMS->moveMountDir("/etc/opt/csm", "$::MOUNT_POINT/etc", 1, $pvals, $bvals, $cvals);
      HAMS->moveMountDir("/var/opt/csm", "$::MOUNT_POINT/var", 1, $pvals, $bvals, $cvals);
      
    }


    NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/data/\*");
    
    HAMS->unMountSharedDisk($pvals, $bvals, $cvals);
    
}

sub migrateBack{
  my ($class, $pvals, $bvals, $cvals) = @_;
  require Cwd;
  my $orig_dir =  Cwd::getcwd();
  
  my @dirs = ("/csminstall", "/etc/opt/csm", "/var/opt/csm");
  my $string = join ', ', @dirs;
  my $copied =0;
  NodeUtils->message('I', 'IMsgCopyingFilesBack', $string);
  foreach my $dir (@dirs){
    my $skip=0;
    if(!-l "$dir" && -d "$dir"){
      my @partition = HAMS->isMounted($pvals, $bvals, $cvals, $dir);
      if(NodeUtils->isLinux() && ($partition[1] =~ "/var/opt/hams/mnt")){
	NodeUtils->runcmd("$::UMOUNT $dir");
	NodeUtils->runcmd("$::RM -rf $dir");
      }
      else{
	NodeUtils->message('I', 'IMsgDirExists', $dir);
	chomp(my $line = <STDIN>);
	if ($line =~ /n|N/)
	  {
	    NodeUtils->message('I', 'IMsgNotRestoreDir', $dir);
	    $skip=1;
	  }
	elsif ($line =~ /y|Y/)
	  {
	    NodeUtils->message('I', 'IMsgRemoveDir', $dir);
	    if(defined $$cvals{'CSMInstallPartition'}){
	      NodeUtils->runcmd("$::RM -rf $dir",-1);
	      NodeUtils->runcmd("$::UMOUNT $dir", -1);
	    }
	    else{
	      NodeUtils->runcmd("$::RM -rf $dir");
	    }
	  }
      }
    }
    if((!$skip) || ((-l "$dir") || (!-e "$dir"))){
      if(-l "$dir"){
	if(unlink($dir)!=1){ NodeUtils->message('E2', 'EMsgUnlinkFailed', $dir, $!); }
      }
      if($dir =~ m/csminstall/){
	if(defined $$cvals{'CSMInstallPartition'}){
	  mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	  NodeUtils->runcmd("$::MOUNT $$cvals{'CSMInstallPartition'} $dir");
	}
	my @output = NodeUtils->runcmd("$::LS -1 $::MOUNT_POINT/csminstall/", -1);
	if((scalar @output > 0) && ($::RUNCMD_RC==0)){
	  if(NodeUtils->isAIX()){
	    NodeUtils->runcmd("$::RM -rf /csminstall", -1);
	    NodeUtils->runcmd("$::CP -Rfph $::MOUNT_POINT/csminstall /");
	  }
	  else{
	    mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	    chdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "chdir", $dir, $!);
	    NodeUtils->runcmd("$::TAR cf - -C $::MOUNT_POINT/csminstall . | $::TAR xvf - ");
	    #check to make sure TAR worked
	  }
	  NodeUtils->runcmd("/usr/bin/diff -r $::MOUNT_POINT/csminstall/ $dir/");
	  NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/csminstall");
	  $copied++;
	}
	else{ #just make the directory
	  mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	}
      }
      elsif($dir =~ m/etc/){
	my @output = NodeUtils->runcmd("$::LS -1 $::MOUNT_POINT/etc/", -1);
	if((scalar @output > 0) && ($::RUNCMD_RC==0)){
	  mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	  chdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "chdir", $dir, $!);
	  NodeUtils->runcmd("$::TAR cf - -C $::MOUNT_POINT/etc . | $::TAR xvf - ");
	  #check to make sure TAR worked
	  NodeUtils->runcmd("/usr/bin/diff -r $::MOUNT_POINT/etc/ $dir/");
	  NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/etc");
	  $copied++;
	}
	else{ #just make the directory
	  mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	}
      }
      elsif($dir =~ m/var/){
	my @output = NodeUtils->runcmd("$::LS -1 $::MOUNT_POINT/var/", -1);
	if((scalar @output > 0) && ($::RUNCMD_RC==0)){
	  mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	  chdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "chdir", $dir, $!);
	  NodeUtils->runcmd("$::TAR cf - -C $::MOUNT_POINT/var . | $::TAR xvf - ");
	  #check to make sure TAR worked
	  NodeUtils->runcmd("/usr/bin/diff -r $::MOUNT_POINT/var/ $dir/");
	  NodeUtils->runcmd("$::RM -rf $::MOUNT_POINT/var");
	  $copied++;
	}
	else{ #just make the directory
	  mkdir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "mkdir", $dir, $!);
	}
      }
    }
  }
  chdir($orig_dir);
  
  if($copied < 3){
    return 1; #did not copy all directories
  }
}


###########################################################
# mountSharedDisk
###########################################################
sub mountSharedDisk{
    my ($class, $pvals, $bvals, $cvals, $dontlocal) = @_;
    if(!HAMS->isMounted($pvals, $bvals, $cvals)){
      if(NodeUtils->isLinux()){
        my $rc = HAMS->check_disk_info($cvals);
        if ($rc eq 1){
          # disk name may be changed
          NodeUtils->message('W', 'EMsgSearchingSharedDisk');
          my $partition = HAMS->get_disk_by_info($::INFO);
          if (!defined $partition){
            NodeUtils->message('E1', 'EMsgNoValidDisk');
          }
          HAMS->store_disk_name($partition, $$cvals{'hostname'});
        }
      }
      #if(NodeUtils->isLinux()){
      #run the echo SCSI commands to add and remove SCSI tiebreaker device
      #echo "scsi remove-single-device 3 0 0 0" > /proc/scsi/scsi
      #echo "scsi add-single-device 3 0 0 0" > /proc/scsi/scsi
      #    my $id =$$cvals{'SCSILockID'};
      #    my $lun =$$cvals{'SCSILockLUN'};
      #    my $host =$$cvals{'SCSILockHOST'};
      #    my $chan = $$cvals{'SCSILockCHAN'};
      #    if($id ne ""){
      #	my $string = "$host $chan $id $lun"; 
      #	NodeUtils->runcmd("echo \"scsi remove-single-device $string\" > /proc/scsi/scsi", -1);
      #	NodeUtils->runcmd("echo \"scsi add-single-device $string\" > /proc/scsi/scsi", -1);
      #    }
      #}
      
	if(NodeUtils->isAIX() && defined $$cvals{'VolumeGroup'}){
	    my $primary = $$pvals{'hostname'};
	    my $backup = $$bvals{'hostname'};

	    #use alarm because sometimes dsh cannot kill child proc
	    $SIG{ALRM} = sub { die "timeout expired\n" };
	    eval {
	      alarm( 15 );
	      NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"/usr/sbin/umount /var/opt/hams/mnt\"", -1);	
	      alarm ( 0 );
	    };
	    alarm ( 0 );
	    
	    eval {
	      alarm( 15 );
	      NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary \"/usr/sbin/varyoffvg $$pvals{'VolumeGroup'}\"", -1);
	      alarm ( 0 );
	    };
	    alarm ( 0 );
	    
	    eval {
	      alarm( 15 );
	      NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $backup \"/usr/sbin/varyoffvg $$bvals{'VolumeGroup'}\"", -1);
	      alarm ( 0 );
	    };
	    alarm ( 0 );

	    NodeUtils->runcmd("/usr/sbin/varyonvg $$cvals{'VolumeGroup'}", -1);
	    if($::RUNCMD_RC !=0 ){
	      my $hdisk = HAMS->get_hdisk($$cvals{'VolumeGroup'});
	      HAMS->break_reservation($hdisk);
	    }
	    NodeUtils->runcmd("/usr/sbin/varyonvg $$cvals{'VolumeGroup'}");
	}

	#mount the shared disk partition to /var/opt/hams/mnt/,
	my $partition = HAMS->get_disk_name($cvals);
	
	if(!-e $::MOUNT_POINT){ mkdir("/var/opt/hams"); mkdir($::MOUNT_POINT); }
	if(NodeUtils->isAIX()){
	    NodeUtils->runcmd("$::MOUNT -o rw $partition $::MOUNT_POINT");
	}
	else{ 
	    NodeUtils->runcmd("$::MOUNT -w $partition $::MOUNT_POINT");
	    if(!defined $dontlocal){
	      if(!-d "/csminstall"){ mkdir("/csminstall");}
	      NodeUtils->runcmd("$::MOUNT --bind $::MOUNT_POINT/csminstall /csminstall");
	      if(!-d "/csminstall"){ mkdir("/var/opt/csm");}
	      NodeUtils->runcmd("$::MOUNT --bind $::MOUNT_POINT/var /var/opt/csm");
	      if(!-d "/csminstall"){ mkdir("/etc/opt/csm");}
	      NodeUtils->runcmd("$::MOUNT --bind $::MOUNT_POINT/etc /etc/opt/csm");
	    }
	}
	$::HAMS_MOUNT=1;
    }
}

###########################################################
# unMountSharedDisk
###########################################################
sub unMountSharedDisk{
    my ($class, $pvals, $bvals, $cvals, $display_errors) = @_;
    if(!defined $display_errors){ $display_errors = -2; }
    my $partition = HAMS->get_disk_name($cvals);

    if(NodeUtils->isLinux()){
      #umount local dirs
      foreach my $dir ("/csminstall", "/var/opt/csm", "/etc/opt/csm"){
        if(HAMS->isMounted($pvals, $bvals, $cvals, $dir)){
          HAMS->unexport($dir);
          NodeUtils->runcmd("$::UMOUNT $dir");
        }
      }
    }
    if(HAMS->isMounted($pvals, $bvals, $cvals)){
      #see if anything is exported from the shared disk
      HAMS->unexport($::MOUNT_POINT);
      NodeUtils->runcmd("$::UMOUNT $::MOUNT_POINT", $display_errors);
      if(!$::RUNCMD_RC){ $::HAMS_MOUNT=0;}
    }
    if(NodeUtils->isAIX() && defined $$cvals{'VolumeGroup'}){
	NodeUtils->runcmd("/usr/sbin/varyoffvg $$cvals{'VolumeGroup'}", 0);
    }
}

###########################################################
# isMounted
# returns 1 if mounted
#         0 if not mounted
###########################################################
sub isMounted{
    my ($class, $pvals, $bvals, $cvals, $dir) = @_;
    HAMS->printDebug("inside HAMS->isMounted\n");
    my $partition;
    if(!defined $dir){ 
      $dir = $::MOUNT_POINT; 
      $partition = HAMS->get_disk_name($cvals);
    }
    my $ismounted = 0;
    #    if($partition =~ m/\w/){
    #	HAMS->printDebug("partition = $partition\n");
    #TODO: fix $partition since we are not always running on primary
    if(($dir eq $::MOUNT_POINT) && $::HAMS_MOUNT){ $ismounted =1; }
    else{
      my @output = NodeUtils->runcmd("/bin/df -P", -1);
      if(scalar @output > 1){
	my $found =0;
	foreach my $line (@output){
	  HAMS->printDebug("DF line =  \"$line\"\n");
	  my ($file_sys, $blocks, $used, $avail, $cap, $mount_point) = split(' ',$line);
	  if ($mount_point =~ m/$dir/){
	    HAMS->printDebug("line \"$line\" matches: $dir\n");
	    $found =1;
	    my ($part, $rest) = split(/\s+/, $line, 2);
	    if($partition !~ m/\w/){ $partition = $part;}
	    last;
	  }
	}
	if($found){
	  $ismounted =1;
	  NodeUtils->message('I', "IMsgDiskDirMounted", $partition, $dir);
	}
      }
    }
    if($dir eq $::MOUNT_POINT){
      HAMS->printDebug("HAMS_MOUNT = $::HAMS_MOUNT\n");
      HAMS->printDebug("ismounted = $ismounted\n");
    }
  
    if($ismounted==1 && !$::HAMS_MOUNT && ($dir eq $::MOUNT_POINT)){ $::HAMS_MOUNT =1; }
    if (wantarray){
      return ($ismounted, $partition);
    }
    else{
      return $ismounted;
    }
}

###########################################################
# printDebug
###########################################################
sub printDebug{
	my ($class, $string) = @_;
	#print "inside debug $::HAMS_DEBUG\n";
	if($::HAMS_DEBUG){
		print "\tHAMS_DEBUG: $string";
	}
}

###########################################################
# loadResource
###########################################################
sub loadResource{

    my $progname = $0;
    my $output = NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 lsrsrc-api -i -s IBM.DmsCtrl::::RemoteShell::SetupRemoteShell::HAMode", -1);
    ($::REMOTE_SHELL, $::SETUP_REMOTE_SHELL, $::HAMODE) = split "::", $output;
   	HAMS->printDebug("$::REMOTE_SHELL, $::SETUP_REMOTE_SHELL, $::HAMODE\n");
   	HAMS->printDebug("progname = $progname\n");
   	if($::HAMODE==0){
		exit 2;
	}
	elsif(($::HAMODE==1) || ($::HAMODE==2)){
		
		if($progname =~ m/Active_MS|Active_CSM|Active_CSM_Dep|Active_CSM_Req|Shared_Disk_Dep|Shared_Disk|Shared_Disk_Req|Save_CSM_Data/){
			HAMS->printDebug("progname \"$progname\" matches an active ms resource\n");
			return 1;
		}
	}
	elsif($::HAMODE==3){
		if($progname =~ m/Inactive_CSM|Gather_CSM_Nodelist/){
				HAMS->printDebug("progname \"$progname\" matches an IN active ms resource\n");
				return 1;
			
			#see if other machine is active ms
			#my @output = NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=2 lsrsrc-api -i -s IBM.DmsCtrl::::NodeNameList::HAMode", -1);
		   # my $inactive=0;
		   # foreach my $line (@output){
			#	chomp $line;
			#	my ($name, $hamode) = split "::", $line;
			#	if($hamode==1){
			#		$inactive=1;
			#	}
			#}

		}
	}

	exit 2;

}

sub listCondResp{
  my %conditions = (
		    "IBM.ManagedNodeChanged" => qq(ResourceClass::IBM.ManagedNode::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.ManagedNode Changed"::Severity::0::ManagementScope::1),
		    "IBM.AssociationChanged" => qq(ResourceClass::IBM.Association::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.Association Changed"::Severity::0::ManagementScope::1),
		    "IBM.ConditionChanged" => qq(ResourceClass::IBM.Condition::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.Condition Changed"::Severity::0::ManagementScope::1),
		    "IBM.EventResponseChanged" => qq(ResourceClass::IBM.EventResponse::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.EventResponse Changed"::Severity::0::ManagementScope::1),
		    "IBM.SensorChanged" => qq(ResourceClass::IBM.Sensor::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.Sensor Changed"::Severity::0::ManagementScope::1),
		    "IBM.NodeGroupChanged" => qq(ResourceClass::IBM.NodeGroup::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.NodeGroup Changed"::Severity::0::ManagementScope::1), 
		    "IBM.DmsCtrlChanged" => qq(ResourceClass::IBM.DmsCtrl::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.DmsCtrl Changed"::Severity::0::ManagementScope::1),
		    "IBM.HwCtrlPointChanged" => qq(ResourceClass::IBM.HwCtrlPoint::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.HwCtrlPoint Changed"::Severity::0::ManagementScope::1),
		    "IBM.DeviceGroupChanged" => qq(ResourceClass::IBM.DeviceGroup::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.DeviceGroup Changed"::Severity::0::ManagementScope::1),
		    "IBM.DeviceHwCtrlChanged" => qq(ResourceClass::IBM.DeviceHwCtrl::EventExpression::"ConfigChanged >= 0"::EventDescription::"IBM.DeviceHwCtrl Changed"::Severity::0::ManagementScope::1),
		    "NIMDatabaseChanged" => qq(ResourceClass::IBM.Sensor::EventExpression::"String!=String\@P"::EventDescription::"NIM Database Changed"::SelectionString::Name=\\"NIMDatabaseChanged\\"::Severity::0::ManagementScope::1),

		);
		
  # These are the definitions of the responses that will be created.  This is a hash in
  # which the key is the name of the response and the value is all the attribute
  # name & value pairs separated by double colons (in the mkrsrc-api format) - in this
  # case only the action array.
  my %responses = (
		   HAMSSaveData => qq(Actions::'{["/opt/csm/csmbin/hams/HAMSSaveData",{127},{0},{86400},"/opt/csm/csmbin/hams/HAMSSaveData",3,0,0,0,{},1]}'),
		   HAMSSaveNIMDB => qq(Actions::'{["/opt/csm/csmbin/hams/HAMSSaveData --nim",{127},{0},{86400},"/opt/csm/csmbin/hams/HAMSSaveData --nim",3,0,0,0,{},1]}'),
		  );
  
  # These are the condition and response pairs that should be activated by default.
  # This is an hash, where the key consists of '"condition" "response"' 
  # The double quotes around condition and response are required, as that is how
  # it is displayed by lscondresp
  my %condresp = (
		  '"IBM.ManagedNodeChanged" "HAMSSaveData"' => 1,
		  '"IBM.AssociationChanged" "HAMSSaveData"' => 1,	
		  '"IBM.ConditionChanged" "HAMSSaveData"' => 1,
		  '"IBM.EventResponseChanged" "HAMSSaveData"' => 1,
		  '"IBM.SensorChanged" "HAMSSaveData"' => 1,
		  '"IBM.NodeGroupChanged" "HAMSSaveData"' => 1,	
		  '"IBM.DmsCtrlChanged" "HAMSSaveData"' => 1,
		  '"IBM.HwCtrlPointChanged" "HAMSSaveData"' => 1,
		  '"IBM.DeviceHwCtrlChanged" "HAMSSaveData"' => 1,
		  '"IBM.DeviceGroupChanged" "HAMSSaveData"' => 1,
		  '"NIMDatabaseChanged" "HAMSSaveNIMDB"' => 1,
		 );
  
  my %sensors = (
		 NIMDatabaseChanged => qq(Command::"/opt/csm/csmbin/mtime /var/adm/nim/nimdb.changed"::UserName::root::RefreshInterval::15),
		);

  if(!NodeUtils->isAIX()){
    #delete sensor, condition, response and association for NIM
    delete $sensors{'NIMDatabaseChanged'};
    delete $conditions{'NIMDatabaseChanged'};
    delete $responses{'HAMSSaveNIMDB'};
    delete $condresp{'"NIMDatabaseChanged" "HAMSSaveNIMDB"'};
  }
  
	return (\%conditions, \%responses, \%condresp, \%sensors);
}

###########################################################
# createCondResp
###########################################################
sub createCondResp{
	
	my ($conditions, $responses, $condresp, $sensors) = HAMS->listCondResp();

	HAMS->queryCondResp($conditions, 'Condition');
	HAMS->queryAssociation($condresp);
	HAMS->queryCondResp($responses, 'EventResponse');
	HAMS->queryCondResp($sensors, 'Sensor');

	HAMS->defineCondResp($sensors, 'Sensor');
	HAMS->defineCondResp($conditions, 'Condition');
	HAMS->defineCondResp($responses, 'EventResponse');
	HAMS->startAssociation($condresp);

}

sub deleteCondResp
{
		my ($conditions, $responses, $condresp) = HAMS->listCondResp();
		
	    HAMS->rmAssociation($condresp);
	    HAMS->removeCondResp($conditions, 'Condition');
	    HAMS->removeCondResp($responses, 'EventResponse');
	
}

sub queryCondResp
  {
	my ($class, $hashref, $resclass) = @_;
	my $outref = HAMS->runCondResp('lsrsrc-api', "-i -D ':|:'", "-s IBM.${resclass}::::Name");   # Get the current list of resources
	foreach my $g (@$outref) { delete $$hashref{$g}; }    # Remove all these entries from our hash
  }
  
sub defineCondResp
{
	my ($class, $hashref, $resclass) = @_;
	if (!scalar(%$hashref)) { return; }      # query() must have deleted them all

	# Build the mkrsrc-api argument string with all the conditions in it
	my $s = '';
	foreach my $k (keys %$hashref)
	  {
		my $attrs = $$hashref{$k};
		$s .= qq( IBM.${resclass}::Name::"$k"::$attrs);
	  }

	HAMS->runCondResp('mkrsrc-api', '', $s);
}

sub removeCondResp
  {
	my ($class, $hashref, $resclass) = @_;
		
	# Use the list of keys in the above hash to delete all the predefined resources which have no associations
	my $output = NodeUtils->runcmd("/usr/sbin/rsct/bin/lscondresp",-1);
    	my @redundant_resources = grep {$output !~ $_} keys(%$hashref);
    	my $where = q/"Name IN ('/ . join("','",@redundant_resources) . q/')"/;
	
	#HAMS->runCondResp('rmrsrc-api', '-i', "-s IBM.${resclass}::$where");
	#NodeUtils->runcmd("/usr/bin/chrsrc-api -s IBM.Condition::\"Name IN ('NodeFullInstallComplete','NodeManaged','NodeChanged')\"::Locked::0", -1);
	#NodeUtils->runcmd("/usr/bin/chrsrc-api -s IBM.EventResponse::\"Name IN ('RunCFMToNode','GatherSSHHostKeys','rconsoleUpdateResponse')\"::Locked::0", -1);
	NodeUtils->runcmd("/usr/bin/chrsrc-api -s IBM.${resclass}::$where::Locked::0");
	my $cmd = "/usr/bin/rmrsrc-api -i -s IBM.${resclass}::$where";
	NodeUtils->runcmd($cmd, -1);
    }

sub rmAssociation
   {
	   my ($class, $hashref) = @_;
	   foreach my $assoc (keys %$hashref){
		system("/usr/sbin/rsct/bin/rmcondresp $assoc 2>/dev/null");
	   }
   } 
   
sub startAssociation
   {
	   my ($class, $hashref) = @_;
	   foreach my $assoc (keys %$hashref){
		system("/usr/sbin/rsct/bin/startcondresp $assoc");
	   }
   }   

#see if any of our associations already exist  
sub queryAssociation
   {
	   my ($class, $hashref) = @_;
	   my @out = `/usr/sbin/rsct/bin/lscondresp 2>/dev/null`;
	   shift @out; #first line is a comment
	   shift @out; #the second line is a comment
	   foreach my $line (@out){
		   chomp $line;
		   my ($condition, $response, $node, $state) = split ' ', $line;
		   my $key = "$condition $response";
		   delete $$hashref{$key};
	   }
   } 

sub runCondResp
  {
	my ($class, $rmccmd, $options, $resources) = @_;
	#$rmccmd = "CT_MANAGEMENT_SCOPE=1 " . $rmccmd;
	my $current_scope = $ENV{'CT_MANAGEMENT_SCOPE'};
	$ENV{'CT_MANAGEMENT_SCOPE'}=1;
	my $outref = NodeUtils->runrmccmd($rmccmd, $options, $resources);
	my $rc = $::RUNCMD_RC;
	$::maxrc = ($rc>$::maxrc ? $rc : $::maxrc);
	$ENV{'CT_MANAGEMENT_SCOPE'}=$current_scope;
	return $outref;
  }

sub determineActive{
  my ($class, $pvals, $bvals) = @_;
  if(defined $HAMS::Active_MS){ return $HAMS::Active_MS; } #cached
  if(!defined $pvals || !defined $bvals){
	  ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
	}
	my $active;
	my $cmd = "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -D ':|:' -i -s IBM.Application::\"Name IN ('Active_CSM')\"::Name::NodeNameList::OpState";
	my @output = NodeUtils->runcmd($cmd,-1);
	if($::RUNCMD_RC!=0){ return 1; }
	foreach my $line (@output){
	        HAMS->printDebug("line = $line\n");
		my ($resource, $nodenamelist, $opstate) = split ':\|:', $line;
		HAMS->printDebug("opstate = $opstate\n");
		if($opstate == 1){
			if($nodenamelist !~ /,/){ #not a list
				$nodenamelist =~ s/{|}//g;
				HAMS->printDebug("nodenamelist = $nodenamelist\n");
				$active = $nodenamelist;		
			}
		}
	}
  $HAMS::Active_MS=$active; #cache it
	return $active;
}

sub determineInactive{
  my ($class, $pvals, $bvals) = @_;
  if(defined $HAMS::Inactive_MS){ return $HAMS::Inactive_MS; } #cached
  if(!defined $pvals || !defined $bvals){
    ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
  }
  my $primary = $$pvals{'hostname'};
  my $backup = $$bvals{'hostname'};
  my $active_ms = HAMS->determineActive($pvals, $bvals);	
  my $inactive_ms;
  if($active_ms eq $primary){ $inactive_ms = $backup; }
  else{ $inactive_ms = $primary; }
  $HAMS::Inactive_MS=$inactive_ms; #cache it
  return $inactive_ms;
}

sub waitAppOffline{
	
	my ($class, $pvals, $bvals, $resource) = @_;
	my $active;
	my $cmd = "/usr/bin/lsrsrc-api -D ':|:' -i -s IBM.Application::\"Name IN (\'$resource\')\"::Name::NodeNameList::OpState";
	my @output = NodeUtils->runcmd($cmd, -1);
	if($::RUNCMD_RC){ return 0; }
	foreach my $line (@output){
		#print "line = $line\n";
		my ($resource, $nodenamelist, $opstate) = split ':\|:', $line;
		#print "opstate = $opstate\n";
		if($opstate == 1){
			if($nodenamelist !~ /,/){ #not a list
				$nodenamelist =~ s/{|}//g;
				#print "nodenamelist = $nodenamelist\n";
				$active = $nodenamelist;		
			}
		}
	}
	if($active){
		return 1;
	}
	else{
		return 0; #offline
	}
}

sub waitRGOffline{
	
	my ($class, $resource) = @_;
	my $active;
	my $cmd = "/usr/bin/lsrsrc-api -D ':|:' -i -s IBM.ResourceGroup::\"Name IN (\'$resource\')\"::Name::OpState";
	my @output = NodeUtils->runcmd($cmd, -1);
	if($::RUNCMD_RC){ return 0; }
	foreach my $line (@output){
		#print "line = $line\n";
		my ($resource, $opstate) = split ':\|:', $line;
		#print "opstate = $opstate\n";
		if($opstate==1 || $opstate==5 || $opstate == 6){ #online, or pending
		  $active=1;
		}
	}
	if($active){
		return 1;
	}
	else{
		return 0; #offline
	}
}

sub createLocalDirs{
    my ($class, $link, $cvals) = @_;
    if(NodeUtils->isLinux()){ return; }
    my $local_dir = "/var/opt/hams";
    my @dirs = ("csminstall.local", "etc.local", "var.local");
    foreach my $d (@dirs){
	if(!-d "$local_dir/$d"){ mkdir("$local_dir/$d"); }
    }
    HAMS->moveDir("/csminstall", "$local_dir/csminstall.local", 1, $cvals);
    HAMS->moveDir("/etc/opt/csm", "$local_dir/etc.local", 1, $cvals);
    HAMS->moveDir("/var/opt/csm", "$local_dir/var.local", 1, $cvals);
}


sub moveDir{
  my ($class, $orig, $new, $move, $cvals) = @_;
  require Cwd;
  my $orig_dir = Cwd::getcwd();

  HAMS->printDebug("inside moveDir: orig = $orig\tnew = $new\tmove = $move\n");
  if(defined $orig && defined $new){
    if(-e "$orig"){
      HAMS->printDebug("$orig exists!\n");
      if(-l "$orig"){ #then maybe its already a symbolic link to $destdir
	my $where = readlink("$orig");
	HAMS->printDebug("$orig is a symbolic link to $where\n");
	$where =~ s/\/\Z//; #remove the last slash because it messes up the comparison
	if($where ne $new){
	  HAMS->printDebug("$where is ne to $new: unlink $orig");
	  unlink("$orig");
	  $move=0;
	}
	else{
	  #this link is already correct;
	  return 0;
	}
      }
      if($move){
	#make sure there are files in there
	my @output = NodeUtils->runcmd("$::LS -1 $orig");
	if(scalar @output > 0){
	  if(NodeUtils->isAIX() && $orig =~ /csminstall/ && $new !~ /csminstall.local/){
	    NodeUtils->runcmd("$::RM -rf $new");
	    NodeUtils->runcmd("$::CP -Rfph $orig /var/opt/hams/mnt");
	  }
	  elsif(NodeUtils->isAIX() && $orig =~ /csminstall/){ #csminstall.local
	    NodeUtils->runcmd("$::RM -rf $new");
	    NodeUtils->runcmd("$::CP -Rfph $orig /var/opt/hams/");
	    NodeUtils->runcmd("$::MV /var/opt/hams/csminstall /var/opt/hams/csminstall.local");
	  }
	  else{
	    mkdir($new);
	    chdir($new);
	    NodeUtils->runcmd("$::TAR cf - -C $orig . | $::TAR xvf - ");
	  }
	  #check to make sure TAR worked
	  NodeUtils->runcmd("/usr/bin/diff -r $orig/ $new/");
	}
      }
      if(($orig eq "/csminstall") && (defined $$cvals{'CSMInstallPartition'})){
	if(NodeUtils->isLinux()){
	  #check if there are running processes.
	  my $output = NodeUtils->runcmd("/usr/sbin/lsof | grep ' /csminstall'", -1);
	  if($output =~ m/\S/){
	    NodeUtils->message('E2', 'EMsgLSOF', $orig, "\"/usr/sbin/lsof | grep \" /csminstall\"\"");
	  }
	}
	NodeUtils->runcmd("$::UMOUNT $orig", 0);
	if($::RUNCMD_RC!=0){ #UMOUNT FAILED
	  NodeUtils->runcmd('I', 'IMsgUmountFailed');
	}
      }
      NodeUtils->runcmd("$::RM -rf $orig"); 
    }
    else{ #doesn't exist but might be a broken link
      NodeUtils->runcmd("$::RM -rf $orig");
    }
    NodeUtils->runcmd("$::LN -s $new/ $orig");
  }

  chdir($orig_dir);

}

sub moveMountDir{
  my ($class, $orig, $new, $move, $pvals, $bvals, $cvals) = @_;
  require Cwd;
  my $orig_dir = Cwd::getcwd();

  HAMS->printDebug("inside moveMountDir: orig = $orig\tnew = $new\tmove = $move\n");
  if(defined $orig && defined $new){
    if(-e "$orig"){
      HAMS->printDebug("$orig exists!\n");
      #see if its mounted
      my @partition = HAMS->isMounted($pvals, $bvals, $cvals, $orig);
      if($partition[1] =~ "/var/opt/hams/mnt"){ return 0; }
      if(-l "$orig"){ #then maybe its already a symbolic link to $destdir
	unlink("$orig");
      }
      elsif($move){
	#make sure there are files in there
	my @output = NodeUtils->runcmd("$::LS -1 $orig");
	if(scalar @output > 0){
	  if(NodeUtils->isAIX() && $orig =~ /csminstall/){ 
	    #this should never happen since this routine is not called on AIX
	    #however I am leaving it in just in case
	    NodeUtils->runcmd("$::RM -rf $new");
	    NodeUtils->runcmd("$::CP -Rfph $orig /var/opt/hams/mnt");
	  }
	  else{
	    mkdir($new);
	    chdir($new);
	    NodeUtils->runcmd("$::TAR cf - -C $orig . | $::TAR xvf - ");
	  }
	}
      }
      if(($orig eq "/csminstall") && (defined $$cvals{'CSMInstallPartition'})){
	if(NodeUtils->isLinux()){
	  #check if there are running processes.
	  my $output = NodeUtils->runcmd("/usr/sbin/lsof | grep ' /csminstall'", -1);
	  if($output =~ m/\S/){
	    NodeUtils->message('E2', 'EMsgLSOF', $orig, "\"/usr/sbin/lsof | grep \" /csminstall\"\"");
	  }
	}
	NodeUtils->runcmd("$::UMOUNT $orig", 0);
	if($::RUNCMD_RC!=0){ #UMOUNT FAILED
	  NodeUtils->runcmd('I', 'IMsgUmountFailed');
	}
      }
      #NodeUtils->runcmd("$::RM -rf $orig"); 
    }
    else{ #doesn't exist but might be a broken link
      unlink($orig);
    }
    if(!-d $orig){ mkdir($orig); } 
    NodeUtils->runcmd("$::MOUNT --bind $new $orig");
  }

  chdir($orig_dir);

}

sub getManagementServerAttr{
    my ($class, $node) = @_;
    if(!defined %HAMS::MSHash){
	my ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
	my $primary = $$pvals{'hostname'};
	my $backup = $$bvals{'hostname'};
	my $active_ms = HAMS->determineActive($pvals, $bvals);	
	my $inactive_ms;
	if($active_ms eq $primary){ $inactive_ms = $backup; }
	else{ $inactive_ms = $primary; }
	my $lsrsrc_cmd = "CT_MANAGEMENT_SCOPE=2 /usr/bin/lsrsrc-api -D ':|:' -i -s IBM.ManagedNode::\"NodeNameList IN ('$inactive_ms')\"::Hostname::ManagementServer";
	my @values = NodeUtils->runcmd($lsrsrc_cmd, -1);
	foreach my $line (@values){
	    my ($hostname, $ms) = split ':\|:', $line;
	    $HAMS::MSHash{$hostname}=$ms;
	}
    }
    return $HAMS::MSHash{$node};
}

sub getInactiveRSCTHostname{
  if(defined $HAMS::LSNODEID){ return $HAMS::LSNODEID; }
	my ($pvals, $bvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
	my $inactive_ms = HAMS->determineInactive($pvals, $bvals);	
	my $output = NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $inactive_ms /usr/sbin/rsct/bin/lsnodeid", -1);
	my ($hostname, $nodeid) = split ": ", $output;
  $HAMS::LSNODEID=$nodeid; #cache for later
	return $nodeid;
}

sub checkHAMSInstalled{

    my ($class, $primary, $backup) = @_;
    
    my $cmd;
    my $isAIX =0;
    if(NodeUtils->isAIX()){
	$cmd = "$::LSLPP -cLq csm.hams";
	$isAIX=1;
    }
    else{ #Linux
	$cmd = "$::RPMCMD -q csm.hams";
    }

    my @output = NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup \"$cmd\"", -1);
    my %installed;
    
    if($isAIX){
	foreach my $line (@output){
	    my ($hostname, $name1, $name2, $version) = split(/:/, $line, 4);
	    if($name1 =~ "csm.hams"){
		$installed{$hostname}=$version;
	    }
	}
    }
    else{
	foreach my $line (@output){
	    my ($hostname, $rpm) = split(/:/, $line, 2);
	    if($rpm =~ "csm.hams-"){
		my ($name, $version, $spin) = split(/-/, $rpm);
		$installed{$hostname}=$version;
	    }
	}

    }
    my @notinstalled;
    foreach my $node ($primary, $backup){
	if(!$installed{$node}){
	    push @notinstalled, $node;
	}
    }
    if(scalar @notinstalled > 0){
	my $nstring = join ",", @notinstalled;
	NodeUtils->message('I', "IMsgNotInstalled", $nstring);
	exit 1;
	#TODO: change to error message
    }
}

sub waitRG
{
    my ($class, $state, $primary, $backup) = @_;
    my @output;
    if($state==1){ print "Waiting for HAMS to start\n"; }
    elsif($state==2){ print "Waiting for HAMS to stop\n"; }
    my $count =1;
    my $active_ms;
    my $inactive_ms;
    my $timeout =200;
    my $ver = $::VERBOSE;
    $::VERBOSE=0;
    my @numnodes = NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 /usr/bin/lsrsrc-api -s IBM.ManagedNode::::Name", -1);
    $::VERBOSE=$ver;
    if($#numnodes > 100){ $timeout = 2 * $#numnodes; }
    while((($active_ms!=$state) || ($inactive_ms!=$state)) && ($count <$timeout)){
	sleep 2;
	print ".";
	$::VERBOSE=0;
	@output = NodeUtils->runcmd("/usr/bin/lsrsrc-api -i -s IBM.ResourceGroup::\"Name IN ('Active_MS','Inactive_MS')\"::Name::OpState 2>&1", -1);
	$::VERBOSE=$ver;
	if($::RUNCMD_RC!=0){
	  $active_ms = $inactive_ms = $state;
	}
	else{
	  chomp @output;
	  foreach my $line (@output){
	    my ($name, $opstate) = split '::', $line;
	    if($opstate==$state){
	      if($name eq "Active_MS"){ $active_ms=$state; }
	      elsif($name eq "Inactive_MS"){ $inactive_ms=$state; }
	    }
	  }
	}
	$count++;
    }
    print "\n";
    if($count==$timeout){ #timed out
	NodeUtils->message('E2', 'EMsgTimedOut', "Active_MS, Inactive_MS"); 
    }
    if($state==1){ 
	NodeUtils->message('I', 'IMsgHAMSStarted');
    }
    elsif($state==2){ 
	NodeUtils->message('I', 'IMsgHAMSStopped');
    }
    return 0; #successful
}

sub waitActive_MS
{
    my ($class, $state, $primary, $backup) = @_;
    my @output;
    my $count =1;
    my $active_ms;
    while(($active_ms!=$state) && ($count <100)){
	print ".";
	sleep 1;
	@output = `lsrsrc-api -i -s IBM.ResourceGroup::\"Name IN ('Active_MS')\"::Name::OpState 2>&1`;
	chomp @output;
	foreach my $line (@output){
	    my ($name, $opstate) = split '::', $line;
	    if($opstate==$state){
		if($name eq "Active_MS"){ $active_ms=$state; }
	    }
	}
	$count++;
    }
    print "\n";
    if($count==200){ #timed out
	NodeUtils->message('E2', 'EMsgTimedOut', "Active_MS, Inactive_MS"); 
    }

    return 0; #successful
}

sub removeCHARMRes{
  my ($class, $primary, $backup)=@_;
  NodeUtils->runcmd("/usr/bin/rmrg -s \"Name IN ('Active_MS','Inactive_MS')\"", -1); 
  NodeUtils->runcmd("/usr/bin/rmrsrc -s \"Name IN ('HAMS_IPAT')\" IBM.ServiceIP", -1);
  NodeUtils->runcmd("/usr/bin/rmequ HAMS_NET_EQU", -1);

  HAMS->rmrsrc_apps($primary);
  HAMS->rmrsrc_apps($backup);

  #make sure the resources were removed
  my @resources = qw(Active_CSM Inactive_CSM Active_CSM_Dep Active_CSM_Req Shared_Disk_Dep Shared_Disk Shared_Disk_Req Save_CSM_Data Public_IP_Dep Public_IP_Req Gather_CSM_Nodelist Active_VLAN_Connection Inactive_VLAN_Connection);
  my $where = NodeUtils->buildWhereInList(\@resources, "Name");
  my @res_defined = NodeUtils->runcmd("/usr/bin/lsrsrc-api -s IBM.Application::${where}::Name", -1);
  if(!$::RUNCMD_RC && (scalar @res_defined > 0)){ #there are some resources left
    #my $string = join " ", @res_defined;
    #NodeUtils->message('E2', 'EMsgResourcesLeft', $string);
    sleep 2;
    HAMS->rmrsrc_apps($primary);
    HAMS->rmrsrc_apps($backup);
    NodeUtils->runcmd("/usr/sbin/rsct/bin/dms/stopdms -s cthats",0);
  }
  
  NodeUtils->runcmd("/usr/bin/rmrsrc -s \"Name IN ('HAMS_IPAT')\" IBM.ServiceIP", -1); 
  
  NodeUtils->runcmd("chrsrc -c IBM.PeerNode OpQuorumTieBreaker=\"Operator\"",-1);

  NodeUtils->runcmd("/usr/bin/rmrsrc -s \"Name IN ('$::TB_NAME')\" IBM.TieBreaker", -1); 
}

#1 = force copy
sub copyLocal{
    my ($class, $force, $pvals, $bvals, $cvals) = @_;
    if(NodeUtils->isLinux()){
	my $last_version = HAMS->getLastVersion();
	my $csmversion = NodeUtils->get_CSMVersion("csm.server");
	if(($force) || ((defined $last_version) && ($last_version ne "") && ($last_version ne $csmversion))){
	    my $distrib = NodeUtils->get_DistributionName();
	    # my $version = NodeUtils->get_DistributionVersion();
	    NodeUtils->runcmd("$::UMOUNT /csminstall");
	    my $dir = "/csminstall/Linux/$distrib/csm/$csmversion/packages/";
	    if( -d $dir){
	      NodeUtils->runcmd("$::CP -a $dir /var/opt/hams/mnt/csminstall/Linux/$distrib/csm/$csmversion/");
	    }
	    NodeUtils->runcmd("$::MOUNT --bind /var/opt/hams/mnt/csminstall /csminstall");
	}
    }
}

sub storeLastVersion{
    
    my $csmversion = NodeUtils->get_CSMVersion("csm.server");
    NodeUtils->runcmd("echo $csmversion > /var/opt/hams/mnt/.last_csmversion");
}

sub getLastVersion{
  if(-e "/var/opt/hams/mnt/.last_csmversion"){
    my $csmversion = NodeUtils->runcmd("$::CAT /var/opt/hams/mnt/.last_csmversion");
    $csmversion =~ s/^\s*//;    # Remove any leading whitespace
    $csmversion =~ s/\s*$//;    # Remove any trailing whitespace
    return $csmversion;
  }
  return "";
}

#log events via CLOG

sub logEvent{
  my ($class, $sccsID, $message, $fields) = @_;
  if(!defined $::clog_ptr){
    $::clog_ptr = CLOG->new(compID => "_HAM",
			    lppID => "HAMS",
			    messageCatalog => "hams.cat",
			    messageSet => "hams",
			    messageMapDir => "/opt/csm/msgmaps"
			   );
  }

  my $return_code = $::clog_ptr->clog_rec(fileName => __FILE__,
					  lineNumber => __LINE__,
					  sccsID => $sccsID,
					  messageType => CLOG::CLOG_INFO_MSG,
					  messageID => $message,
					  fields => $fields);
  if($return_code !=0){
    NodeUtils->message('E', 'EMsgCLOGError', $message, $return_code);
  }
}

sub resource_started{
  my ($class, $resource) = @_;
  my $message = "IMsgStartedResource";
  NodeUtils->message("LI", $message, $resource);
  my $sccsID   = "1.264.1.2";
  my @fields = ("$resource");
  HAMS->logEvent($sccsID, $message, \@fields);

}

sub resource_stopped{
  my ($class, $resource) = @_;
  my $message = "IMsgStoppedResource";
  NodeUtils->message("LI", $message, $resource);
  my $sccsID   = "1.264.1.2";
  my @fields = ("$resource");
  HAMS->logEvent($sccsID, $message, \@fields);
}

sub start_hwctrlrm{
  shift @_; #class
  my $nodes = join ",", @_;
  my $cmd;
  if($nodes !~ m/\w/){ #run on localhost
    $cmd = "/opt/csm/csmbin/hams/hwctrl --start";
  }
  else{ #run through dsh
    $cmd = "$::DSH -t $HAMS::TIMEOUT -vn $nodes /opt/csm/csmbin/hams/hwctrl --start";
  }
  NodeUtils->runcmd($cmd, 0);
  if($::RUNCMD_RC){ return 1; }
  return 0;
}

sub stop_hwctrlrm{
  shift @_; #class
  my $nodes = join ",", @_;
  my $cmd;
  if($nodes !~ m/\w/){ #run on localhost
    $cmd = "/opt/csm/csmbin/hams/hwctrl --stop";
  }
  else{ #run through dsh
    $cmd = "$::DSH -t $HAMS::TIMEOUT -vn $nodes /opt/csm/csmbin/hams/hwctrl --stop";
  }
  NodeUtils->runcmd($cmd, 0);
  if($::RUNCMD_RC){ return 1; }
  return 0;
}

sub runact_hwctrlrm{

	my $rc = NetworkUtils->verify_atd;	
	if ($rc)
	{
		NodeUtils->runcmd("/bin/echo \"CT_MANAGEMENT_SCOPE=1 /usr/bin/runact-api -c IBM.HwCtrlPoint::::HAModeActive::NewHAMode::1\"");
	}else
	{	
		NodeUtils->runcmd("/bin/echo \"CT_MANAGEMENT_SCOPE=1 /usr/bin/runact-api -c IBM.HwCtrlPoint::::HAModeActive::NewHAMode::1\" | at now");
	}	

}

sub rmrsrc_apps{
  my ($class, $ms) = @_;
  NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $ms \"/opt/csm/csmbin/hams/rmapplications Active_CSM Inactive_CSM Active_CSM_Dep Active_CSM_Req Shared_Disk_Dep Shared_Disk Shared_Disk_Req Save_CSM_Data Public_IP_Dep Public_IP_Req Gather_CSM_Nodelist Active_VLAN_Connection Inactive_VLAN_Connection \"", -1);
  
}

#this is run on the inactive MS to make sure the nodes
#can be listed
sub accept_csm_license{
  my ($class, $pvals, $bvals, $cvals) = @_;
  #first see if we can already list the nodes
  my $need_license = HAMS->check_dmsrm();
  if(!$need_license){ return 0; }

  #otherwise we need to accept the license
  #dcp the files from the other ms to /var/opt/csm
  my $active_ms; #can't get active_ms from resources since they aren't online yet
  if($$cvals{'hostname'} eq $$pvals{'hostname'}){ $active_ms = $$bvals{'hostname'}; }
  else{ $active_ms = $$pvals{'hostname'}; }


  if($active_ms && (-e "/var/opt/csm")){
    
    # NodeUtils->runcmd("/opt/csm/bin/dcp -rn $active_ms /var/opt/csm/ /var/opt/csm");  #DCP isn't working right
    my $found =0;
    my $count =0; 
    while (($found == 0)&&($count <5)){
      my $output = NodeUtils->runcmd("LANG=C $::REMOTE_SHELL $active_ms \"$::LS -d /var/opt/csm\"");
      HAMS->printDebug("$output \n");
      if(($output =~ /\/var\/opt\/csm/) && ($output !~ /not found/)){
	$found=1;
      }
      else{
	sleep 1;
	$count++;
      }
    }
    $count =0;
    if($found){
      while($need_license && ($count <5)){
	if($::REMOTECOPYCMD =~ /scp/){ NodeUtils->runcmd("$::REMOTECOPYCMD -r $active_ms:/var/opt/csm/ /var/opt/", -1); }
	else{ NodeUtils->runcmd("$::REMOTECOPYCMD -r $active_ms:/var/opt/csm/ /var/opt/csm/", -1); }

	#now set the correct LicenseProductVersion
	my $licdir = "/var/opt/csm/lic";
	my $lumfile   = 'csmlum.lic';
	if(!-e "$licdir/$lumfile"){
	  $lumfile = 'csmlum.tab';
	}
	$lumfile = "$licdir/$lumfile";
	my $lumversion = NodeUtils->runcmd("/bin/grep ProductVersion $lumfile",0);
	if($::RUNCMD_RC == 0){
	  $lumversion =~ s/ProductVersion=//;
	  my $origenv =$ENV{'CT_MANAGEMENT_SCOPE'};
          $ENV{'CT_MANAGEMENT_SCOPE'}=1;
	  NodeUtils->runrmccmd('chrsrc-api','-i',qq(-s IBM.DmsCtrl::::LicenseProductVersion::$lumversion));
	  $ENV{'CT_MANAGEMENT_SCOPE'}=$origenv;
	}
	NodeUtils->runcmd("CT_MANAGEMENT_SCOPE=1 /usr/bin/refrsrc-api -c IBM.DmsCtrl");
	#NodeUtils->runrmccmd('refrsrc-api', '-c IBM.DmsCtrl');
	$need_license = HAMS->check_dmsrm(); #check to see its working now
	$count++;
      }
    }

  }
  
  if(!$need_license){ return 0; }
  else{ return 1; } #could not accept license

}

sub check_dmsrm{

  NodeUtils->runcmd("/opt/csm/bin/lsnode", -1);
  if(!$::RUNCMD_RC){ return 0; } #can already list the nodes
  else{ return $::RUNCMD_RC; }
}

#logs the monitor exit code through clog if a file exists
sub monitor_debug{
  my ($class, $resource, $exit_code) = @_;
  if(-e "/var/opt/hams/resources/debug" || ($exit_code!=1 && $exit_code!=2 && $exit_code!=0)){
    my $message = "IMsgMonitoredResource";
    NodeUtils->message("lV", $message, $resource, $exit_code); #display only -- do not log
    my $sccsID   = "1.264.1.2";
    my @fields = ("$resource", $exit_code);
    HAMS->logEvent($sccsID, $message, \@fields);  
    $? = $exit_code; #make sure this wasn't changed by CLOG
  }
}

#usage HAMS->get_hdisk($$cvals{'VolumeGroup'});
#AIX only
sub get_hdisk{
  my ($class, $vg) = @_;

  my $vid = NodeUtils->runcmd("/usr/sbin/getlvodm -v $vg");
  my @disks = NodeUtils->runcmd("/usr/sbin/getlvodm -w $vid");
  my ($id, $hdisk) = split " ", $disks[0];

  return "/dev/$hdisk";
}

#usage: HAMS->break_reservation($hdisk);
#AIX only
sub break_reservation{
  my ($class, $disk)= @_;
  my ($nothing, $diskname) = split "\/dev\/", $disk;

  #determine disk type, code taken from HACMP
  my $parent = NodeUtils->runcmd("/usr/sbin/lsdev -Cc disk -l $diskname -F parent");
  my $pddvln = NodeUtils->runcmd("/usr/sbin/lsdev -Cc disk -l $diskname -F PdDvLn");
  my $disktype;
  if($parent != m/\w/){ print "error getting parent type of disk $disk\n"; return 2; }
  if($parent =~ m/scsi/){ $disktype="SCSIDISK"; }
  elsif($parent =~ m/ssar/){ $disktype="SSA"; }
  else{
    NodeUtils->runcmd("/bin/odmget -q \"name = $parent AND PdDvLn = disk/fdar/array\" CuDv", -1);
    if(!$::RUNCMD_RC){
      #   Either Dolphin or Flute.  They behave differently, but have
      #   nearly identical ODM entries.
      my $daclist = NodeUtils->runcmd("/bin/odmget -q \"name = $parent AND attribute = all_controller\" CuAt | grep 'value =' | cut -f2 -d'\"'");
      if($daclist !~ m/\w/){ $disktype="FCPARRAY"; } #daclist is empty
      else{ $disktype="FLUTE"; }
    }
    elsif($parent =~ m/dar/){ $disktype="ARRAY"; }
    elsif($parent =~ m/dpo/){ $disktype="DPO"; }
    elsif($parent =~ m/fscsi/){ $disktype="FSCSI"; }
  }

  $ENV{'VERBOSE_LOGGING'} = "high";

  if($disktype eq "FLUTE"){
    NodeUtils->runcmd("/opt/csm/csmbin/hams/cl_flutereset $disk");
  }
  elsif($disktype eq "SSA"){
    NodeUtils->runcmd("/opt/csm/csmbin/hams/cl_scdiskrsrv $disk");
  }
  else{
    print "disktype $disktype not supported\n";
    return 2;
  }

}

sub try_write_shared_disk{
  my ($class, $pvals, $bvals, $cvals);
  my $pid;
  #check to see if an old child process is already running, if so don't fork, just return 3
  my @processes = NodeUtils->runcmd("/bin/ps -eo pid,args", -1);
  my @checkprocs = grep(/hamscheckdisk/, @processes);
  if(scalar @checkprocs > 0){
    HAMS->printDebug("grep is true: @checkprocs");
    $::TOUCH_FAILED=1;
    exit 3;
  }

  #try writing to the shared disk -- set an alarm so we don't wait forever
  $SIG{ALRM} = sub { die "timeout expired\n" };
  eval {
    alarm( 10 );
    $::TOUCH_FAILED=1;
    HAMS->printDebug("spawing child\n");
    $pid = fork();
    if( ! defined $pid )
      {
	undef $::TOUCH_FAILED;
	NodeUtils->message('E1', 'EMsgCantFork');
      }
    if( $pid == 0 )
      {
	# Child - do some work
	undef $::TOUCH_FAILED;
	NodeUtils->runcmd("/opt/csm/csmbin/hams/hamscheckdisk");
	exit 0;
    }
    HAMS->printDebug("Waiting for child\n");
    my $child = waitpid( -1, 0 ); # Wait for any child
    my $status = $? >> 8;
    HAMS->printDebug("Parent reaped $child, returned $status\n");
    if($status==3){
      $::TOUCH_FAILED=1;
      alarm ( 0 );
      exit 3;
    }
    undef $::TOUCH_FAILED; #looks like the shared disk is reachable
    alarm ( 0 );
  };
  kill 9, $pid;
  # If the message from the eval is from the timeout subroutine,
  # we could not touch the file in the given amount of time
  # Write a message and exit.
  #
  if ( $@ =~ /timeout expired/ ) { $::TOUCH_FAILED=1; exit 3; }
  delete $SIG{ALRM};
}

sub displayLicenseAgreement
{
    my $tab = shift;
    my $rc = HAMS->invokeLAP();
    if ($rc == 9)
    {

        # Tell the user the license was accepted.
        # a return code of 9 from invokeLAP means
        # that the user accepted it.
        #NodeUtils->msg($::MSGS, 'I', 'EMsgACCEPTED');
        NodeUtils->message('I', 'IMsgAccepted');
    }
    else
    {

        # Tell the user the license was not accepted.
        #NodeUtils->msg($::MSGS, 'W3', 'EMsgNOT_ACCEPTED');
        NodeUtils->message('W3', 'EMsgNOT_Accepted');
    }

    # return true or false if the license was accepted or not.
    return ($rc == 9);
}

#--------------------------------------------------------------------------------

=head3    invokeLAP

        Invoke the LAP tool and display the CSM license agreements.
        The LAP tool will be invoked and copy the LA and LI files to

            /var/opt/csm/<VRnumber>/<type>/license/<lang>.txt
        Where:
                <VRnumber> = CSM Version (1.3, etc)
                <type>     = tryandbuy or full, depending upon the license.
                <lang>     = one of the 17 supported LAP languages.

        Arguments:
                A string - either: "tab" or "full"

        Returns: one of [ 3 | 9 | 127 ]
                If everything ran successful, then LAP will return 9,
        otherwise, if the user returns 3, the license was not
        accepted.  If something went wrong with the LAP tool,
        then an error of 127 will be returned.
        Error:
                127

        Example: Doesn't appear to be used in csm/install

        Comments: none

=cut

#--------------------------------------------------------------------------------

sub invokeLAP
{
    my ($class, $licType) = @_;
    my ($csmVersion, $laDir, $strCmd, $rc);
    my $laptool = "/opt/csm/install/lapapp.jar";

    # make sure that $licType is either tab or full
    $licType="full";

    # get the CSM version:
    #$csmVersion = InstallUtils->get_CSMVersion("csm.server");
    #$csmVersion = sprintf "%.1f", $csmVersion;                  # e.g. "1.3"

    $csmVersion="1.4";

    # check that the LA and LI files are in place:
    if (NodeUtils->isLinux())
    {
      my $arch = InstallUtils->get_PkgArchitecture();
      if($arch =~ m/ppc/){ #pLinux
	$laDir = "/opt/csm/hams/$csmVersion/pLinux/license/$licType";
      }
      else{ #xLinux
	$laDir = "/opt/csm/hams/$csmVersion/xLinux/license/$licType";
      }
    }
    else                                                        # AIX
    {
        $laDir = "/opt/csm/hams/$csmVersion/AIX/license/$licType";
    }

    unless (-f "$laDir/LA_en")
    {
        NodeUtils->messageFromCat(
                                  'csmInstall.cat',     $::MSGMAPPATH,
                                  'csminstall',         'E127',
                                  'EMsgCANT_READ_FILE', "$laDir/LA_en"
                                  );
    }

    # check that the LAP tool is in place:
    unless (-f "$laptool")
    {
        NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall',
                                  'E127', 'EMsgCANT_READ_FILE', $laptool);
    }

    # Everything should be in place now, so let's call LAP and  construct the command:

    if (NodeUtils->isLinux())
    {

        #try to find java
        my $found_java = 0;
        my ($java, $rt, $i18n);
        my @output = NodeUtils->runcmd("$::LS -1d /opt/IBMJava\*");
        while (($found_java != 1) && (@output))
        {
            my $dir = shift @output;
            $dir =~ s/^\s*//;    # Remove any leading whitespace
            $dir =~ s/\s*$//;    # Remove any trailing whitespace
            my $tmp_java = "$dir/jre/bin/java";
            my $tmp_rt   = "$dir/jre/lib/rt.jar";
            my $tmp_i18n = "$dir/jre/lib/i18n.jar";
            if (-e "$tmp_java")
            {
                $found_java = 1;
                $java       = $tmp_java;
                $rt         = $tmp_rt;
                $i18n       = $tmp_i18n;
            }
        }
        if ($found_java)
        {
            $strCmd = "$java -cp /opt/csm/install/lapapp.jar:${rt}:${i18n}:";
        }
        else
        {    #try the old way
                # check that Java is installed... and use the right one.
            if (-d "/opt/IBMJava2-141")
            {
                $strCmd = "/opt/IBMJava2-141/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-141/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-141/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-131")
            {
                $strCmd = "/opt/IBMJava2-131/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-131/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-131/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-13")
            {
                $strCmd = "/opt/IBMJava2-13/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-13/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-13/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-ppc-141")
            {
                $strCmd = "/opt/IBMJava2-ppc-141/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-141/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-141/jre/lib/i18n.jar:";
            }
            elsif (-d "/opt/IBMJava2-ppc-131")
            {
                $strCmd = "/opt/IBMJava2-ppc-131/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-131/jre/lib/rt.jar:";
                $strCmd .= "/opt/IBMJava2-ppc-131/jre/lib/i18n.jar:";
            }
            elsif (-d "/usr/lib/IBMJava2-1.3.1")
            {
                $strCmd = "/usr/lib/IBMJava2-1.3.1/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/usr/lib/IBMJava2-1.3.1/jre/lib/rt.jar:";
                $strCmd .= "/usr/lib/IBMJava2-1.3.1/jre/lib/i18n.jar:";
            }
            else    # java isn't installed maybe
            {
                my $string = "IBMJava2-13[1]";
                NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH,
                           'csminstall', 'E127', 'EMsgMissingPackage', $string);
            }
        }
    }
    else
    {               # AIX
                    #try to find java
        my $found_java = 0;
        my ($java, $rt, $i18n);
        my @output = NodeUtils->runcmd("$::LS -1d /usr/java\*");
        while (($found_java != 1) && (@output))
        {
            my $dir = shift @output;
            $dir =~ s/^\s*//;    # Remove any leading whitespace
            $dir =~ s/\s*$//;    # Remove any trailing whitespace
            my $tmp_java = "$dir/jre/bin/java";
            my $tmp_rt   = "$dir/jre/lib/rt.jar";
            my $tmp_i18n = "$dir/jre/lib/i18n.jar";
            if (-e "$tmp_java")
            {
                $found_java = 1;
                $java       = $tmp_java;
                $rt         = $tmp_rt;
                $i18n       = $tmp_i18n;
            }
        }
        if ($found_java)
        {
            $strCmd = "$java -cp /opt/csm/install/lapapp.jar:${rt}:${i18n}:";
        }
        else
        {
            if (-d "/usr/java141")
            {
                $strCmd = "/usr/java141/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/usr/java141/jre/lib/rt.jar:";
                $strCmd .= "/usr/java141/jre/lib/i18n.jar:";
            }
            else
            {
                $strCmd = "/usr/java131/jre/bin/java -cp ";
                $strCmd .= "/opt/csm/install/lapapp.jar:";
                $strCmd .= "/usr/java131/jre/lib/rt.jar:";
                $strCmd .= "/usr/java131/jre/lib/i18n.jar:";
            }
        }
    }

    # this part of the command is common in both AIX and Linux:
    $strCmd .= " com.ibm.lex.lapapp.LAP ";
    $strCmd .= "-l $laDir ";
    $strCmd .= "-s /var/opt/hams/$csmVersion/$licType ";
    $strCmd .= "-text_only";
    $rc = system("$strCmd");
    $rc = $rc >> 8;

    # kill processes if they exist:
    if (NodeUtils->isLinux)
    {
        foreach (`ps -efl --columns 300 | grep lapapp | awk '{print \$4}'`)
        {
            chomp $_;
            `kill -9 $_ 2>/dev/null`;
        }
    }

    return $rc;
}

#check to see if there are any symlinked files in /cfmroot
sub check_cfm_links
  {
    my ($class, $path, $silent) = @_;
    if(!defined $path){
      $path = "/cfmroot";
    }
  my %badlinks;
  my $file;
  if(!-d $path){ return; }
  my $dir;
  opendir($dir, $path) or NodeUtils->message('E2', 'EMsgPerlFailed', "opendir", $path, $!);
  while ($file = readdir($dir))
    {
      my $wholename = join '/', $path, $file;
      if ($file ne '.' and $file ne '..')
        {
	  if(-l $wholename){ #if its a link
	    my $link = readlink($wholename);
	    if($link !~ m/cfmroot/){
	      $badlinks{$wholename} = $link;
	    }
	  }
	  elsif(-d $wholename){
	    my $child = HAMS->check_cfm_links($wholename, 1);
	    map{ $badlinks{$_}=$child->{$_} } keys %{$child};
	  }
	}
    }
  closedir($dir) or NodeUtils->message('E2', 'EMsgPerlFailed', "closedir", $path, $!);
    if(!defined $silent){
      my @files = keys %badlinks;
      if(scalar @files > 0){
	NodeUtils->message('E', 'EMsgCFMSymLink');
	foreach my $f (@files){
	  print "$f -> $badlinks{$f}\n";
	}
	print "\n";
      }
    }
    return( \%badlinks );
}

#check the validity of the shared disk
sub check_disk{
    my ($class, $pvals, $bvals, $cvals) = @_;
    if(NodeUtils->isLinux()){
      #first see if its mounted, if it is return
      if(HAMS->isMounted($pvals, $bvals, $cvals)){
	return 0;
      }
      my $disk = $$cvals{'PartitionDeviceName'};
      my $cmd = "/sbin/fsck -np $disk";
      if (NodeUtils->get_DistributionName() eq "SLES" && NodeUtils->get_DistributionVersion() eq "9") {$cmd = "/sbin/fsck -n $disk";}
      # in the case of AIX, the fisrt function will return undef 

      my $output = NodeUtils->runcmd($cmd, 0);
      my $fsck_error = $::RUNCMD_RC;
      if($fsck_error != 0){
	NodeUtils->message('E1', 'EMsgFsckError', $cmd, $output);
      }
    }
}

sub mkrg_mkrel{
  my ($class, $pvals, $bvals, $cvals) = @_;

  #make resource groups
  #Inactive_MS IBM.ResourceGroup Contains the names of all the resources that are required on the inactive management server
  #Active_MS IBM.ResourceGroup Contains the names of all the resources that are required on the active management server
  NodeUtils->runcmd("/usr/bin/mkrg -l Collocated -p 5 Active_MS");
  NodeUtils->runcmd("/usr/bin/mkrg -l Collocated Inactive_MS");

  if($::IPAT==1){
      #make IBM.ServiceIP resources
      my $nodestring =  "\"{'$$pvals{'hostname'}','$$bvals{'hostname'}'}\"";
      NodeUtils->runcmd("/usr/bin/mkrsrc IBM.ServiceIP Name=\"HAMS_IPAT\" NodeNameList=$nodestring IPAddress=$$cvals{'PublicIPAddress'} NetMask=$$cvals{'PublicIPNetmask'} ProtectionMode=0");
      NodeUtils->runcmd("/usr/bin/mkequ HAMS_NET_EQU IBM.NetworkInterface:$$pvals{'PublicIPAdapter'}:$$pvals{'hostname'},$$bvals{'PublicIPAdapter'}:$$bvals{'hostname'}");
      NodeUtils->runcmd("/usr/bin/addrgmbr -g Active_MS IBM.ServiceIP:HAMS_IPAT IBM.Application:Public_IP_Dep IBM.Application:Public_IP_Req ");
      NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Equivalency:HAMS_NET_EQU -S IBM.ServiceIP:HAMS_IPAT HAMS_IPAT_DependsOn");
  }
  
 
  NodeUtils->runcmd("/usr/bin/addrgmbr -g Active_MS IBM.Application:Active_CSM IBM.Application:Active_CSM_Dep IBM.Application:Active_CSM_Req IBM.Application:Shared_Disk_Dep IBM.Application:Shared_Disk IBM.Application:Shared_Disk_Req IBM.Application:Save_CSM_Data IBM.Application:Active_VLAN_Connection");
 
  NodeUtils->runcmd("/usr/bin/addrgmbr -g Inactive_MS IBM.Application:Inactive_CSM IBM.Application:Gather_CSM_Nodelist IBM.Application:Inactive_VLAN_Connection");

  #AC and IC relationships
  NodeUtils->runcmd("/usr/bin/mkrel -p Collocated -o IfNotOffline -S IBM.Application:Active_CSM -G IBM.Application:Inactive_CSM  HAMS_ACIC_Collocated");
  NodeUtils->runcmd("/usr/bin/mkrel -p AntiCollocated -G IBM.Application:Active_CSM -S  IBM.Application:Inactive_CSM HAMS_ICAC_Anti_Collocated");

  #AC relationships inside Active_MS
  #DependsOn
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Shared_Disk -S IBM.Application:Active_CSM HAMS_ACSD_DependsOn");
  if($::IPAT){
  	NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.ServiceIP:HAMS_IPAT -S IBM.Application:Active_CSM HAMS_ACPI_DependsOn");
  }

  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_VLAN_Connection -S IBM.Application:Active_CSM HAMS_ACAVC_DependsOn");
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_CSM -S IBM.Application:Save_CSM_Data HAMS_SVDAC_DependsOn");
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_CSM_Dep -S IBM.Application:Active_CSM HAMS_ACACD_DependsOn");
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Active_CSM -S IBM.Application:Active_CSM_Req HAMS_ACRAC_DependsOn");
  
  #Shared disk relationships
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Shared_Disk_Dep -S IBM.Application:Shared_Disk HAMS_SDSDD_DependsOn");
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Shared_Disk -S IBM.Application:Shared_Disk_Req HAMS_SDRSD_DependsOn");

  #Inactive CSM relationships
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Inactive_CSM -S IBM.Application:Gather_CSM_Nodelist HAMS_GCNIC_DependsOn");
  NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Inactive_VLAN_Connection -S IBM.Application:Inactive_CSM HAMS_ICIVC_DependsOn");

  if($::IPAT){
    NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.Application:Public_IP_Dep -S IBM.ServiceIP:HAMS_IPAT HAMS_HIPID_DependsOn");
    NodeUtils->runcmd("/usr/bin/mkrel -p DependsOn -G IBM.ServiceIP:HAMS_IPAT -S IBM.Application:Public_IP_Req HAMS_PIRHI_DependsOn");
  }
}

#get a lock so that only 1 csm command can be run at a time 
sub block_csm_cmd
  {
      my $file = "/var/opt/hams/resources/.blockCSMlock"; #lock a file in /var so it doesn't need to be removed
      open (BLOCKCSM, ">$file") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
      
      #use an alarm to timeout if we cannot get the lock in a reasonable amount of time
      my $cantlock =0;
      eval { 
	local $SIG{ALRM} = sub { die "alarm clock restart" };
	alarm 60;               # schedule alarm in 600 seconds 
	eval { 
	  if(!flock(BLOCKCSM,LOCK_EX))
	    {
	      $cantlock=1;
	    }
	};
	alarm 0;                # cancel the alarm
      };
      alarm 0;               # race condition protection
      die if $@ && $@ !~ /alarm clock restart/; # reraise
      if($cantlock==1){ die "cannot get lock on $file\n"; }

      return 0;
}

#release lock
sub unblock_csm_cmd
  {
     close(BLOCKCSM);
}

#see if both MS are time synced
sub checkTime{
  my ($class, $primary, $backup)= @_;

  if(!defined $primary || !defined $backup){
    return 1;
  }

  my $cmd = '"perl -e \'print time();\'"';
  $cmd = "$::DSH -t $HAMS::TIMEOUT -vn $primary,$backup " . $cmd;
  my @output = NodeUtils->runcmd($cmd);
  my @times;
  foreach my $line (@output){
    my ($name, $time) = split /: /, $line;
    push @times, $time;
  }
  my $diff = $times[0] - $times[1];
  if(abs $diff > 60){
    NodeUtils->message('E2', 'EMsgNotTimeSynced');
  }
  return 0;
}

sub unexport{
  my ($class, $dir) = @_;
  my @exported = NodeUtils->runcmd("$::EXPORTFS", -1);
  if(!$::RUNCMD_RC && (scalar @exported > 0 )){
    #stuff is exported, look through it to see if we need to unexport anything
    foreach my $line (@exported){
      if($line =~ m/$dir/){ #this is a shared disk directory
	if(NodeUtils->isAIX()){
	  NodeUtils->runcmd("$::EXPORTFS -i -u $line", 0);
	}
	else{
	  NodeUtils->runcmd("$::EXPORTFS -u \*:$line", 0);
	}
      }
    }
  }
}

sub offlineInactive_MS{
  my ($class) = @_;

  NodeUtils->runcmd("/usr/bin/chrg -o Offline Inactive_MS",-1);

  my ($pvals, $bvals, $cvals) = HAMS->getAttributes($::HAMSDEF);
  while(HAMS->waitAppOffline($pvals, $bvals, "Inactive_CSM")){ sleep 1; }
  while(HAMS->waitAppOffline($pvals, $bvals, "Gather_CSM_Nodelist")){ sleep 1; }
  while(HAMS->waitAppOffline($pvals, $bvals, "Inactive_VLAN_Connection")){ sleep 1; }

  if(-e "/var/opt/hams/resources/Inactive_CSM"){
    NodeUtils->runcmd("/bin/rm -f /var/opt/hams/resources/Inactive_CSM", 0);
  }
}

sub remove_HAMS_links{
  my @dirs = ("/etc/opt/csm","/var/opt/csm","/csminstall");
  foreach my $dir (@dirs){
    if (-l $dir){
      my $where = readlink("$dir");
      HAMS->printDebug("$dir is a symbolic link to $where \n");
      if ($where !~ /\.local/){
        unlink("$dir");		
        NodeUtils->runcmd("$::MKDIR -p $dir");
      }
    }
  }
}

#----------------------------------------------------------------

=head3 get_info

    get host, chan, id, lun and sequence number of shared disk on Linux

    Arguments:
      $arg: it is partition device name

    Return:
      a scalar that contains 4 disk attributes and sequence number (successful)
      1         (failed)

=cut

#----------------------------------------------------------------
sub get_info{
  my ($class, $partition, $msvals, $inactive_vals) = @_;

  if (NodeUtils->isAIX()){
    return 1;
  }

  my $cmd = "/opt/csm/csmbin/hams/scsinum";
  my $info = NodeUtils->runcmd("$cmd $partition",0);
  if ($::RUNCMD_RC != 0){
    return 1;
  }
  my ($disk_name, $attrs) = split(' ', $info);
  $partition =~ s/\D+//g;
  $attrs .= " $partition";
  if(!defined $inactive_vals){
    return "$$msvals{'hostname'}#$attrs";
  }
  else{
    my $first = "$$msvals{'hostname'}#$attrs";
    my $i_part = $$inactive_vals{'PartitionDeviceName'};
    my $hostname = $$inactive_vals{'hostname'};
    $info = NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $hostname $cmd $i_part",0);
    if ($::RUNCMD_RC != 0){
      return 1;
    }
    my $dsh_hostname;
    ($dsh_hostname, $disk_name, $attrs) = split(' ', $info);
    $partition =~ s/\D+//g;
    $attrs .= " $partition";
    return "$first\n$$inactive_vals{'hostname'}#$attrs";
  }

}

#----------------------------------------------------------------

=head get_disk_name

    get disk name

    on Linux,
        it searched the file '/var/opt/hams/sharedDiskInfo' first.
        if it does not contain local disk name, this function will
        return $$ms{'PartitionDeviceName'}
    on AIX,
        it return $$ms{'PartitionDeviceName'}

    Arguments:
        $ms     reference to a hash that contains management server

    Return;
        $disk   partition disk name

    Example:
        HAMS->get_disk_name($pvals);

=cut

#-----------------------------------------------------------------
sub get_disk_name{
  my ($class, $ms) = @_;
  my $disk = $$ms{'PartitionDeviceName'};
  my $file = "/var/opt/hams/sharedDiskInfo";

  if (NodeUtils->isAIX()){
    return $disk;
  }
  if (!-f $file){
    return $disk;
  }

  my ($isRemote, $info_file);
  my ($pvals, $bvals, $cvals) = HAMS->getAttributes("/var/opt/hams/hamsdef");
  if ($$cvals{'hostname'} eq $$ms{'hostname'}){
    $info_file = $file;
  }else{
    $info_file = "/var/opt/hams/remote_sharedDiskInfo";
    NodeUtils->runcmd("$::DSH -t $HAMS::TIMEOUT -vn $$ms{'hostname'} \"/opt/csm/bin/dcp -n $$cvals{'hostname'} $file $info_file\"", -1);
    if ($::RUNCMD_RC != 0){
      return $disk;
    }
    $isRemote ++;
  }
  #my $content = NodeUtils->runcmd("$::CAT $info_file",0);
  open(FILE, "<$info_file") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $info_file, $!);
  my ($ms_name, $disk_info, $disk_name);
  while(my $line = <FILE>){
    chomp $line;
    ($ms_name, $disk_info, $disk_name) = split('#', $line);
    if($ms_name eq $$ms{'hostname'}){ last; }
  }
  close(FILE);
  if ($isRemote){
    unlink $info_file;
  }
  if ($disk_name =~ m/\S+/g){
    $disk_name =~ s/\s+//g;
    return $disk_name;
  }
  return $disk;
}

#-----------------------------------------------------------------

=head get_disk_by_info

    get device partition name of shared disk on Linux

    Arguments:
        $info:    a scalar that contains comma separated host, chan,
                  id, lun and sequence number of shared disk

    Return:
        a scalar that contains device partition name (when successful)
        undef   (when failed)

=cut

#-----------------------------------------------------------------
sub get_disk_by_info{
  my ($class, $info) = @_;
  my (@disks);

  if (NodeUtils->isAIX()){
    return;
  }

  # list all disk partitions
  my $file = "/proc/partitions";
  my @output = NodeUtils->runcmd("$::CAT $file",0);
  foreach my $line (@output){
    if ($line =~ /sd\D\d/g){
      my $device = "\/dev\/";
      my ($major, $minor, $blocks, $disk) = split(' ', $line);
      $device .= $disk;
      push @disks, $device;
    }
  }
  if (!scalar(@disks)){
    return;
  }elsif (scalar(@disks) eq 1){
    return $disks[0];
  }

  # select the partitions with suitable info
  my $cmd = "/opt/csm/csmbin/hams/scsinum ";
  my $disk_str = join(' ', @disks);
  my ($attrs, $seq_num) = split(' ', $info);
  my @output = NodeUtils->runcmd("$cmd $disk_str",0);
  foreach my $line (@output){
    if ($line =~ $attrs){
      my ($disk, $attr) = split(' ', $line);
      if ($disk =~ $seq_num){
        return $disk;
      }
    }
  }
  return;  # if no available partition is found, return unde
}

#------------------------------------------------------------------

=head store_disk_name

    store local partition device name of shared disk on Linux
    this info will not be transfered to remote MS

    Arguments:
        $partition: a scalar that contains partition device name

    Return:

=cut

#------------------------------------------------------------------
sub store_disk_name{
  my ($class, $partition, $ms) = @_;
  my $file = "/var/opt/hams/sharedDiskInfo";

  # get original contents first
  my @file_contents;
  open(FILE, "<$file") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
  while(my $line = <FILE>){ push @file_contents, $line; }
  close(FILE);

  my @new_file_contents;
  foreach my $line (@file_contents){
    chomp $line;
    my ($msname, $info, $disk_name) = split('#', $line);
    if($msname eq $ms){
      $line = "$ms#$info#$partition";
    }
    push @new_file_contents, "$line\n";
  }

  # update this file
  open(INFOFILE,">$file") || NodeUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
  print INFOFILE @new_file_contents;
  close INFOFILE;
}


#-----------------------------------------------------------------------

=head check_disk_info

    check whether partition device name corresponds to its info
    on Linux

    Arguments:
        $ms     reference to a hash that contains current MS definition

    Return:
        0       share disk meets its original info
        1       match failed

        $::INFO the info of possible partitions to be searched

=cut

#------------------------------------------------------------------------
sub check_disk_info{
  my ($class, $cvals) = @_;
  my $file = "/var/opt/hams/sharedDiskInfo";

  # if the file storing info does not contain enough message,
  # we should skip the info examination
  if (!-f $file){
    NodeUtils->message('W', 'EMsgInvalidInfoFile');
    return 0;
  }
  open(FILE, "<$file") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
  my ($ms_name, $disk_info, $disk_name);
  while(my $line = <FILE>){
    chomp $line;
    ($ms_name, $disk_info, $disk_name) = split('#', $line);
    if($ms_name eq $$cvals{'hostname'}){ last; }
  }
  close(FILE);
  if ($disk_info !~ /\S+/g){
    NodeUtils->message('W', 'EMsgInvalidInfoFile');
    return 0;
  }

  my $disk = $$cvals{'PartitionDeviceName'};
  my $msdisk_info = "$ms_name#$disk_info";
  my $target_info = HAMS->get_info($disk, $cvals);
  if ($msdisk_info eq $target_info){
    return 0;
  }

  $::INFO = $disk_info;
  return 1;
}

#----------------------------------------------------------------

=head store_disk_info

    store host, chan, id, lun and the sequence number of shared disk
    on both management servers on Linux

    Arguments:
        $ms        reference to a hash that contains current MS info
        $inactive  host name of inactive managment server

    Return:
        0          succeed
        1          fail
=cut

#----------------------------------------------------------------
sub store_disk_info{
  my ($class, $ms, $inactive) = @_;

  my $disk = $$ms{'PartitionDeviceName'};
  my $info = HAMS->get_info($disk, $ms, $inactive);
  if ($info eq 1){
    return 1;
  }

  # create local definition file and copy it to inactive MS
  my $file = "/var/opt/hams/sharedDiskInfo";
  open (HANDLE, ">$file") or NodeUtils->message('E2', 'EMsgPerlFailed', "open", $file, $!);
  print HANDLE $info;
  close HANDLE;
  NodeUtils->runcmd("/opt/csm/bin/dcp -n $$inactive{'hostname'} $file $file",0);
  return 0;
}

1;
